@@ -5,9 +5,10 @@
  */
 
 #include "config.h"
-#include "adw-action-row.h"
+#include "adw-action-row-private.h"
 
-#include "adw-macros-private.h"
+#include "adw-marshalers.h"
+#include "adw-widget-utils-private.h"
 
 /**
  * AdwActionRow:
@@ -46,6 +47,11 @@
  *
  * It contains subnodes `label.title` and `label.subtitle` representing
  * respectively the title label and subtitle label.
+ *
+ * `AdwActionRow` can use the
+ * [`.property`](style-classes.html#property-rows) style class to emphasize
+ * the row subtitle instead of the row title, which is useful for
+ * displaying read-only properties.
  */
 
 typedef struct
@@ -62,6 +68,7 @@ typedef struct
 
   int title_lines;
   int subtitle_lines;
+  gboolean subtitle_selectable;
   GtkWidget *activatable_widget;
   GBinding *activatable_binding;
 } AdwActionRowPrivate;
@@ -82,6 +89,7 @@ enum {
   PROP_ACTIVATABLE_WIDGET,
   PROP_TITLE_LINES,
   PROP_SUBTITLE_LINES,
+  PROP_SUBTITLE_SELECTABLE,
   LAST_PROP,
 };
 
@@ -101,6 +109,47 @@ string_is_not_empty (AdwActionRow *self,
   return string && string[0];
 }
 
+static void
+pressed_cb (AdwActionRow *self,
+            int           n_press,
+            double        x,
+            double        y,
+            GtkGesture   *gesture)
+{
+  AdwActionRowPrivate *priv = adw_action_row_get_instance_private (self);
+  GtkWidget *picked;
+  GtkEditable *delegate;
+
+  picked = gtk_widget_pick (GTK_WIDGET (self), x, y, GTK_PICK_DEFAULT);
+
+  if (picked != GTK_WIDGET (self) &&
+      picked != priv->header &&
+      picked != GTK_WIDGET (priv->prefixes) &&
+      picked != GTK_WIDGET (priv->suffixes)) {
+    gtk_gesture_set_state (gesture, GTK_EVENT_SEQUENCE_DENIED);
+
+    return;
+  }
+
+  if (!GTK_IS_EDITABLE (self)) {
+    gtk_gesture_set_state (gesture, GTK_EVENT_SEQUENCE_DENIED);
+    return;
+  }
+
+  delegate = gtk_editable_get_delegate (GTK_EDITABLE (self));
+
+  while (delegate && !GTK_IS_TEXT (delegate))
+    delegate = gtk_editable_get_delegate (delegate);
+
+  if (!delegate || !GTK_IS_TEXT (delegate)) {
+    gtk_gesture_set_state (gesture, GTK_EVENT_SEQUENCE_DENIED);
+    return;
+  }
+
+  gtk_text_grab_focus_without_selecting (GTK_TEXT (delegate));
+  gtk_gesture_set_state (gesture, GTK_EVENT_SEQUENCE_CLAIMED);
+}
+
 static void
 row_activated_cb (AdwActionRow  *self,
                   GtkListBoxRow *row)
@@ -152,6 +201,9 @@ adw_action_row_get_property (GObject    *object,
   case PROP_TITLE_LINES:
     g_value_set_int (value, adw_action_row_get_title_lines (self));
     break;
+  case PROP_SUBTITLE_SELECTABLE:
+    g_value_set_boolean (value, adw_action_row_get_subtitle_selectable (self));
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
@@ -181,6 +233,9 @@ adw_action_row_set_property (GObject      *object,
   case PROP_TITLE_LINES:
     adw_action_row_set_title_lines (self, g_value_get_int (value));
     break;
+  case PROP_SUBTITLE_SELECTABLE:
+    adw_action_row_set_subtitle_selectable (self, g_value_get_boolean (value));
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
@@ -226,7 +281,7 @@ adw_action_row_class_init (AdwActionRowClass *klass)
   klass->activate = adw_action_row_activate_real;
 
   /**
-   * AdwActionRow:subtitle: (attributes org.gtk.Property.get=adw_action_row_get_subtitle org.gtk.Property.set=adw_action_row_set_subtitle)
+   * AdwActionRow:subtitle:
    *
    * The subtitle for this row.
    *
@@ -239,7 +294,7 @@ adw_action_row_class_init (AdwActionRowClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwActionRow:icon-name: (attributes org.gtk.Property.get=adw_action_row_get_icon_name org.gtk.Property.set=adw_action_row_set_icon_name)
+   * AdwActionRow:icon-name:
    *
    * The icon name for this row.
    *
@@ -248,15 +303,15 @@ adw_action_row_class_init (AdwActionRowClass *klass)
   props[PROP_ICON_NAME] =
     g_param_spec_string ("icon-name", NULL, NULL,
                          "",
-                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_DEPRECATED);
 
   /**
-   * AdwActionRow:activatable-widget: (attributes org.gtk.Property.get=adw_action_row_get_activatable_widget org.gtk.Property.set=adw_action_row_set_activatable_widget)
+   * AdwActionRow:activatable-widget:
    *
    * The widget to activate when the row is activated.
    *
    * The row can be activated either by clicking on it, calling
-   * [method@ActionRow.activate], or via mnemonics in the title or the subtitle.
+   * [method@ActionRow.activate], or via mnemonics in the title.
    * See the [property@PreferencesRow:use-underline] property to enable
    * mnemonics.
    *
@@ -269,7 +324,7 @@ adw_action_row_class_init (AdwActionRowClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwActionRow:title-lines: (attributes org.gtk.Property.get=adw_action_row_get_title_lines org.gtk.Property.set=adw_action_row_set_title_lines)
+   * AdwActionRow:title-lines:
    *
    * The number of lines at the end of which the title label will be ellipsized.
    *
@@ -282,7 +337,7 @@ adw_action_row_class_init (AdwActionRowClass *klass)
                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwActionRow:subtitle-lines: (attributes org.gtk.Property.get=adw_action_row_get_subtitle_lines org.gtk.Property.set=adw_action_row_set_subtitle_lines)
+   * AdwActionRow:subtitle-lines:
    *
    * The number of lines at the end of which the subtitle label will be
    * ellipsized.
@@ -295,6 +350,20 @@ adw_action_row_class_init (AdwActionRowClass *klass)
                       0,
                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
+  /**
+   * AdwActionRow:subtitle-selectable:
+   *
+   * Whether the user can copy the subtitle from the label.
+   *
+   * See also [property@Gtk.Label:selectable].
+   *
+   * Since: 1.3
+   */
+  props[PROP_SUBTITLE_SELECTABLE] =
+    g_param_spec_boolean ("subtitle-selectable", NULL, NULL,
+                          FALSE,
+                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
   g_object_class_install_properties (object_class, LAST_PROP, props);
 
   /**
@@ -307,9 +376,13 @@ adw_action_row_class_init (AdwActionRowClass *klass)
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_LAST,
                   0,
-                  NULL, NULL, NULL,
+                  NULL, NULL,
+                  adw_marshal_VOID__VOID,
                   G_TYPE_NONE,
                   0);
+  g_signal_set_va_marshaller (signals[SIGNAL_ACTIVATED],
+                              G_TYPE_FROM_CLASS (klass),
+                              adw_marshal_VOID__VOIDv);
 
   gtk_widget_class_set_template_from_resource (widget_class,
                                                "/org/gnome/Adwaita/ui/adw-action-row.ui");
@@ -321,6 +394,7 @@ adw_action_row_class_init (AdwActionRowClass *klass)
   gtk_widget_class_bind_template_child_private (widget_class, AdwActionRow, title);
   gtk_widget_class_bind_template_child_private (widget_class, AdwActionRow, title_box);
   gtk_widget_class_bind_template_callback (widget_class, string_is_not_empty);
+  gtk_widget_class_bind_template_callback (widget_class, pressed_cb);
 }
 
 static void
@@ -387,6 +461,7 @@ adw_action_row_add_prefix (AdwActionRow *self,
 
   g_return_if_fail (ADW_IS_ACTION_ROW (self));
   g_return_if_fail (GTK_IS_WIDGET (widget));
+  g_return_if_fail (gtk_widget_get_parent (widget) == NULL);
 
   priv = adw_action_row_get_instance_private (self);
 
@@ -409,6 +484,7 @@ adw_action_row_add_suffix (AdwActionRow *self,
 
   g_return_if_fail (ADW_IS_ACTION_ROW (self));
   g_return_if_fail (GTK_IS_WIDGET (widget));
+  g_return_if_fail (gtk_widget_get_parent (widget) == NULL);
 
   priv = adw_action_row_get_instance_private (self);
 
@@ -447,7 +523,7 @@ adw_action_row_remove (AdwActionRow *self,
 }
 
 /**
- * adw_action_row_get_subtitle: (attributes org.gtk.Method.get_property=subtitle)
+ * adw_action_row_get_subtitle:
  * @self: an action row
  *
  * Gets the subtitle for @self.
@@ -467,7 +543,7 @@ adw_action_row_get_subtitle (AdwActionRow *self)
 }
 
 /**
- * adw_action_row_set_subtitle: (attributes org.gtk.Method.set_property=subtitle)
+ * adw_action_row_set_subtitle:
  * @self: an action row
  * @subtitle: the subtitle
  *
@@ -495,7 +571,7 @@ adw_action_row_set_subtitle (AdwActionRow *self,
 }
 
 /**
- * adw_action_row_get_icon_name: (attributes org.gtk.Method.get_property=icon-name)
+ * adw_action_row_get_icon_name:
  * @self: an action row
  *
  * Gets the icon name for @self.
@@ -517,7 +593,7 @@ adw_action_row_get_icon_name (AdwActionRow *self)
 }
 
 /**
- * adw_action_row_set_icon_name: (attributes org.gtk.Method.set_property=icon-name)
+ * adw_action_row_set_icon_name:
  * @self: an action row
  * @icon_name: (nullable): the icon name
  *
@@ -546,7 +622,7 @@ adw_action_row_set_icon_name (AdwActionRow *self,
 }
 
 /**
- * adw_action_row_get_activatable_widget: (attributes org.gtk.Method.get_property=activatable-widget)
+ * adw_action_row_get_activatable_widget:
  * @self: an action row
  *
  * Gets the widget activated when @self is activated.
@@ -579,14 +655,14 @@ activatable_widget_weak_notify (gpointer  data,
 }
 
 /**
- * adw_action_row_set_activatable_widget: (attributes org.gtk.Method.set_property=activatable-widget)
+ * adw_action_row_set_activatable_widget:
  * @self: an action row
  * @widget: (nullable): the target widget
  *
  * Sets the widget to activate when @self is activated.
  *
  * The row can be activated either by clicking on it, calling
- * [method@ActionRow.activate], or via mnemonics in the title or the subtitle.
+ * [method@ActionRow.activate], or via mnemonics in the title.
  * See the [property@PreferencesRow:use-underline] property to enable mnemonics.
  *
  * The target widget will be activated by emitting the
@@ -641,7 +717,7 @@ adw_action_row_set_activatable_widget (AdwActionRow *self,
 }
 
 /**
- * adw_action_row_get_title_lines: (attributes org.gtk.Method.get_property=title-lines)
+ * adw_action_row_get_title_lines:
  * @self: an action row
  *
  * Gets the number of lines at the end of which the title label will be
@@ -663,7 +739,7 @@ adw_action_row_get_title_lines (AdwActionRow *self)
 }
 
 /**
- * adw_action_row_set_title_lines: (attributes org.gtk.Method.set_property=title-lines)
+ * adw_action_row_set_title_lines:
  * @self: an action row
  * @title_lines: the number of lines at the end of which the title label will be ellipsized
  *
@@ -695,7 +771,7 @@ adw_action_row_set_title_lines (AdwActionRow *self,
 }
 
 /**
- * adw_action_row_get_subtitle_lines: (attributes org.gtk.Method.get_property=subtitle-lines)
+ * adw_action_row_get_subtitle_lines:
  * @self: an action row
  *
  * Gets the number of lines at the end of which the subtitle label will be
@@ -717,7 +793,7 @@ adw_action_row_get_subtitle_lines (AdwActionRow *self)
 }
 
 /**
- * adw_action_row_set_subtitle_lines: (attributes org.gtk.Method.set_property=subtitle-lines)
+ * adw_action_row_set_subtitle_lines:
  * @self: an action row
  * @subtitle_lines: the number of lines at the end of which the subtitle label will be ellipsized
  *
@@ -748,6 +824,55 @@ adw_action_row_set_subtitle_lines (AdwActionRow *self,
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_SUBTITLE_LINES]);
 }
 
+/**
+ * adw_action_row_get_subtitle_selectable:
+ * @self: a `AdwActionRow`
+ *
+ * Gets whether the user can copy the subtitle from the label
+ *
+ * Returns: whether the user can copy the subtitle from the label
+ *
+ * Since: 1.3
+ */
+gboolean
+adw_action_row_get_subtitle_selectable (AdwActionRow *self)
+{
+  AdwActionRowPrivate *priv = adw_action_row_get_instance_private (self);
+
+  g_return_val_if_fail (ADW_IS_ACTION_ROW (self), FALSE);
+
+  return priv->subtitle_selectable;
+}
+
+/**
+ * adw_action_row_set_subtitle_selectable:
+ * @self: a `AdwActionRow`
+ * @subtitle_selectable: `TRUE` if the user can copy the subtitle from the label
+ *
+ * Sets whether the user can copy the subtitle from the label
+ *
+ * See also [property@Gtk.Label:selectable].
+ *
+ * Since: 1.3
+ */
+void
+adw_action_row_set_subtitle_selectable (AdwActionRow *self,
+                                        gboolean      subtitle_selectable)
+{
+  AdwActionRowPrivate *priv = adw_action_row_get_instance_private (self);
+
+  g_return_if_fail (ADW_IS_ACTION_ROW (self));
+
+  subtitle_selectable = !!subtitle_selectable;
+
+  if (priv->subtitle_selectable == subtitle_selectable)
+    return;
+
+  priv->subtitle_selectable = subtitle_selectable;
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_SUBTITLE_SELECTABLE]);
+}
+
 /**
  * adw_action_row_activate:
  * @self: an action row
@@ -761,3 +886,24 @@ adw_action_row_activate (AdwActionRow *self)
 
   ADW_ACTION_ROW_GET_CLASS (self)->activate (self);
 }
+
+void
+adw_action_row_set_expand_suffixes (AdwActionRow *self,
+                                    gboolean      expand)
+{
+  AdwActionRowPrivate *priv = adw_action_row_get_instance_private (self);
+
+  g_return_if_fail (ADW_IS_ACTION_ROW (self));
+
+  expand = !!expand;
+
+  if (expand) {
+    gtk_widget_set_hexpand (GTK_WIDGET (priv->title_box), FALSE);
+    gtk_label_set_natural_wrap_mode (priv->title, GTK_NATURAL_WRAP_NONE);
+    gtk_label_set_natural_wrap_mode (priv->subtitle, GTK_NATURAL_WRAP_NONE);
+  } else {
+    gtk_widget_set_hexpand (GTK_WIDGET (priv->title_box), TRUE);
+    gtk_label_set_natural_wrap_mode (priv->title, GTK_NATURAL_WRAP_INHERIT);
+    gtk_label_set_natural_wrap_mode (priv->subtitle, GTK_NATURAL_WRAP_INHERIT);
+  }
+}
