@@ -79,8 +79,10 @@ typedef struct
   gboolean use_subtitle;
   gboolean enable_search;
 
+  gboolean uses_default_factory;
   GtkListItemFactory *factory;
   GtkListItemFactory *list_factory;
+  GtkListItemFactory *header_factory;
   GListModel *model;
   GListModel *filter_model;
   GtkSelectionModel *selection;
@@ -88,6 +90,7 @@ typedef struct
   GtkSelectionModel *current_selection;
 
   GtkExpression *expression;
+  GtkStringFilterMatchMode search_match_mode;
 } AdwComboRowPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (AdwComboRow, adw_combo_row, ADW_TYPE_ACTION_ROW)
@@ -98,10 +101,12 @@ enum {
   PROP_SELECTED_ITEM,
   PROP_MODEL,
   PROP_FACTORY,
+  PROP_HEADER_FACTORY,
   PROP_LIST_FACTORY,
   PROP_EXPRESSION,
   PROP_USE_SUBTITLE,
   PROP_ENABLE_SEARCH,
+  PROP_SEARCH_MATCH_MODE,
   LAST_PROP,
 };
 
@@ -133,12 +138,27 @@ selection_changed (AdwComboRow *self)
 {
   AdwComboRowPrivate *priv = adw_combo_row_get_instance_private (self);
   guint selected;
+  GtkFilter *filter;
 
   if (!GTK_IS_SINGLE_SELECTION (priv->selection))
     return;
 
   selected = gtk_single_selection_get_selected (GTK_SINGLE_SELECTION (priv->selection));
 
+  /* reset the filter so positions are 1-1 */
+  filter = gtk_filter_list_model_get_filter (GTK_FILTER_LIST_MODEL (priv->filter_model));
+  if (GTK_IS_STRING_FILTER (filter))
+    gtk_string_filter_set_search (GTK_STRING_FILTER (filter), "");
+  gtk_single_selection_set_selected (GTK_SINGLE_SELECTION (priv->popup_selection), selected);
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_SELECTED]);
+}
+
+static void
+selection_item_changed (AdwComboRow *self)
+{
+  AdwComboRowPrivate *priv = adw_combo_row_get_instance_private (self);
+
   if (priv->use_subtitle) {
     if (g_list_model_get_n_items (G_LIST_MODEL (priv->current_selection)) > 0) {
       GtkListItem *item = g_list_model_get_item (G_LIST_MODEL (priv->current_selection), 0);
@@ -153,9 +173,6 @@ selection_changed (AdwComboRow *self)
     }
   }
 
-  gtk_single_selection_set_selected (GTK_SINGLE_SELECTION (priv->popup_selection), selected);
-
-  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_SELECTED]);
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_SELECTED_ITEM]);
 }
 
@@ -174,9 +191,14 @@ static void
 row_activated_cb (AdwComboRow *self)
 {
   AdwComboRowPrivate *priv = adw_combo_row_get_instance_private (self);
+  GtkFilter *filter;
 
   gtk_popover_popdown (GTK_POPOVER (priv->popover));
 
+  /* reset the filter so positions are 1-1 */
+  filter = gtk_filter_list_model_get_filter (GTK_FILTER_LIST_MODEL (priv->filter_model));
+  if (GTK_IS_STRING_FILTER (filter))
+    gtk_string_filter_set_search (GTK_STRING_FILTER (filter), "");
   adw_combo_row_set_selected (self, gtk_single_selection_get_selected (GTK_SINGLE_SELECTION (priv->popup_selection)));
 }
 
@@ -197,16 +219,19 @@ static void
 update_filter (AdwComboRow *self)
 {
   AdwComboRowPrivate *priv = adw_combo_row_get_instance_private (self);
-  
-  if (priv->filter_model) {
-      GtkFilter *filter;
 
-      if (priv->expression) 
-        filter = GTK_FILTER (gtk_string_filter_new (gtk_expression_ref (priv->expression)));
-      else 
-        filter = GTK_FILTER (gtk_every_filter_new ());
-      gtk_filter_list_model_set_filter (GTK_FILTER_LIST_MODEL (priv->filter_model), filter);
-      g_object_unref (filter);
+  if (priv->filter_model) {
+    GtkFilter *filter;
+
+    if (priv->expression) {
+      filter = GTK_FILTER (gtk_string_filter_new (gtk_expression_ref (priv->expression)));
+      gtk_string_filter_set_match_mode (GTK_STRING_FILTER (filter), priv->search_match_mode);
+    } else {
+      filter = GTK_FILTER (gtk_every_filter_new ());
+    }
+
+    gtk_filter_list_model_set_filter (GTK_FILTER_LIST_MODEL (priv->filter_model), filter);
+    g_object_unref (filter);
   }
 }
 
@@ -270,7 +295,10 @@ setup_item (GtkSignalListItemFactory *factory,
   gtk_widget_set_valign (label, GTK_ALIGN_CENTER);
   gtk_box_append (GTK_BOX (box), label);
 
-  icon = gtk_image_new_from_icon_name ("object-select-symbolic");
+  icon = g_object_new (GTK_TYPE_IMAGE,
+                       "accessible-role", GTK_ACCESSIBLE_ROLE_PRESENTATION,
+                       "icon-name", "object-select-symbolic",
+                       NULL);
   gtk_box_append (GTK_BOX (box), icon);
 
   gtk_list_item_set_child (list_item, box);
@@ -293,20 +321,33 @@ selected_item_changed (AdwComboRow *self,
     gtk_widget_set_opacity (icon, 0.0);
 }
 
+static void
+root_changed (GtkWidget   *box,
+              GParamSpec  *pspec,
+              AdwComboRow *self)
+{
+  AdwComboRowPrivate *priv = adw_combo_row_get_instance_private (self);
+  GtkWidget *icon;
+
+  icon = gtk_widget_get_last_child (box);
+
+  if (gtk_widget_get_ancestor (box, GTK_TYPE_POPOVER) == GTK_WIDGET (priv->popover))
+    gtk_widget_set_visible (icon, TRUE);
+  else
+    gtk_widget_set_visible (icon, FALSE);
+}
+
 static void
 bind_item (GtkSignalListItemFactory *factory,
            GtkListItem              *list_item,
            AdwComboRow              *self)
 {
-  AdwComboRowPrivate *priv = adw_combo_row_get_instance_private (self);
   gpointer item;
   GtkWidget *box;
-  GtkWidget *icon;
   char *repr;
 
   item = gtk_list_item_get_item (list_item);
   box = gtk_list_item_get_child (list_item);
-  icon = gtk_widget_get_last_child (box);
 
   repr = get_item_representation (self, item);
 
@@ -318,14 +359,13 @@ bind_item (GtkSignalListItemFactory *factory,
     g_critical ("Either AdwComboRow:factory or AdwComboRow:expression must be set");
   }
 
-  if (gtk_widget_get_ancestor (box, GTK_TYPE_POPOVER) == GTK_WIDGET (priv->popover)) {
-    gtk_widget_set_visible (icon, TRUE);
-    g_signal_connect (self, "notify::selected-item",
-                      G_CALLBACK (selected_item_changed), list_item);
-    selected_item_changed (self, NULL, list_item);
-  } else {
-    gtk_widget_set_visible (icon, FALSE);
-  }
+  g_signal_connect (self, "notify::selected-item",
+                    G_CALLBACK (selected_item_changed), list_item);
+  selected_item_changed (self, NULL, list_item);
+
+  g_signal_connect (box, "notify::root",
+                    G_CALLBACK (root_changed), self);
+  root_changed (box, NULL, self);
 
   g_clear_pointer (&repr, g_free);
 }
@@ -335,7 +375,32 @@ unbind_item (GtkSignalListItemFactory *factory,
              GtkListItem              *list_item,
              AdwComboRow              *self)
 {
+  GtkWidget *box;
+
+  box = gtk_list_item_get_child (list_item);
+
   g_signal_handlers_disconnect_by_func (self, selected_item_changed, list_item);
+  g_signal_handlers_disconnect_by_func (box, root_changed, self);
+}
+
+static void
+set_factory (AdwComboRow        *self,
+             GtkListItemFactory *factory,
+             gboolean            is_default)
+{
+  AdwComboRowPrivate *priv = adw_combo_row_get_instance_private (self);
+
+  if (!g_set_object (&priv->factory, factory))
+    return;
+
+  gtk_list_view_set_factory (priv->current, factory);
+
+  priv->uses_default_factory = is_default;
+
+  if (priv->list_factory == NULL)
+    gtk_list_view_set_factory (priv->list, factory);
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_FACTORY]);
 }
 
 static void
@@ -347,7 +412,7 @@ set_default_factory (AdwComboRow *self)
   g_signal_connect (factory, "bind", G_CALLBACK (bind_item), self);
   g_signal_connect (factory, "unbind", G_CALLBACK (unbind_item), self);
 
-  adw_combo_row_set_factory (self, factory);
+  set_factory (self, factory, TRUE);
 
   g_object_unref (factory);
 }
@@ -373,6 +438,9 @@ adw_combo_row_get_property (GObject    *object,
   case PROP_FACTORY:
     g_value_set_object (value, adw_combo_row_get_factory (self));
     break;
+  case PROP_HEADER_FACTORY:
+    g_value_set_object (value, adw_combo_row_get_header_factory (self));
+    break;
   case PROP_LIST_FACTORY:
     g_value_set_object (value, adw_combo_row_get_list_factory (self));
     break;
@@ -385,6 +453,9 @@ adw_combo_row_get_property (GObject    *object,
   case PROP_ENABLE_SEARCH:
     g_value_set_boolean (value, adw_combo_row_get_enable_search (self));
     break;
+  case PROP_SEARCH_MATCH_MODE:
+    g_value_set_enum (value, adw_combo_row_get_search_match_mode (self));
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
@@ -411,6 +482,9 @@ adw_combo_row_set_property (GObject      *object,
   case PROP_LIST_FACTORY:
     adw_combo_row_set_list_factory (self, g_value_get_object (value));
     break;
+  case PROP_HEADER_FACTORY:
+    adw_combo_row_set_header_factory (self, g_value_get_object (value));
+    break;
   case PROP_EXPRESSION:
     adw_combo_row_set_expression (self, gtk_value_get_expression (value));
     break;
@@ -420,6 +494,9 @@ adw_combo_row_set_property (GObject      *object,
   case PROP_ENABLE_SEARCH:
     adw_combo_row_set_enable_search (self, g_value_get_boolean (value));
     break;
+  case PROP_SEARCH_MATCH_MODE:
+    adw_combo_row_set_search_match_mode (self, g_value_get_enum (value));
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
@@ -436,6 +513,7 @@ adw_combo_row_dispose (GObject *object)
 
   if (priv->selection) {
     g_signal_handlers_disconnect_by_func (priv->selection, selection_changed, self);
+    g_signal_handlers_disconnect_by_func (priv->selection, selection_item_changed, self);
     g_signal_handlers_disconnect_by_func (priv->selection, model_changed, self);
   }
 
@@ -445,6 +523,7 @@ adw_combo_row_dispose (GObject *object)
   g_clear_object (&priv->current_selection);
   g_clear_object (&priv->factory);
   g_clear_object (&priv->list_factory);
+  g_clear_object (&priv->header_factory);
   g_clear_object (&priv->filter_model);
 
   g_clear_object (&priv->model);
@@ -496,7 +575,7 @@ adw_combo_row_class_init (AdwComboRowClass *klass)
   row_class->activate = adw_combo_row_activate;
 
   /**
-   * AdwComboRow:selected: (attributes org.gtk.Property.get=adw_combo_row_get_selected org.gtk.Property.set=adw_combo_row_set_selected)
+   * AdwComboRow:selected:
    *
    * The position of the selected item.
    *
@@ -509,7 +588,7 @@ adw_combo_row_class_init (AdwComboRowClass *klass)
                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwComboRow:selected-item: (attributes org.gtk.Property.get=adw_combo_row_get_selected_item)
+   * AdwComboRow:selected-item:
    *
    * The selected item.
    */
@@ -519,7 +598,7 @@ adw_combo_row_class_init (AdwComboRowClass *klass)
                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * AdwComboRow:model: (attributes org.gtk.Property.get=adw_combo_row_get_model org.gtk.Property.set=adw_combo_row_set_model)
+   * AdwComboRow:model:
    *
    * The model that provides the displayed items.
    */
@@ -529,7 +608,7 @@ adw_combo_row_class_init (AdwComboRowClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwComboRow:factory: (attributes org.gtk.Property.get=adw_combo_row_get_factory org.gtk.Property.set=adw_combo_row_set_factory)
+   * AdwComboRow:factory:
    *
    * Factory for populating list items.
    *
@@ -542,7 +621,19 @@ adw_combo_row_class_init (AdwComboRowClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwComboRow:list-factory: (attributes org.gtk.Property.get=adw_combo_row_get_list_factory org.gtk.Property.set=adw_combo_row_set_list_factory)
+   * AdwComboRow:header-factory:
+   *
+   * The factory for creating header widgets for the popup.
+   *
+   * Since: 1.6
+   */
+  props[PROP_HEADER_FACTORY] =
+    g_param_spec_object ("header-factory", NULL, NULL,
+                         GTK_TYPE_LIST_ITEM_FACTORY,
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
+  /**
+   * AdwComboRow:list-factory:
    *
    * The factory for populating list items in the popup.
    *
@@ -554,7 +645,7 @@ adw_combo_row_class_init (AdwComboRowClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwComboRow:expression: (type GtkExpression) (attributes org.gtk.Property.get=adw_combo_row_get_expression org.gtk.Property.set=adw_combo_row_set_expression)
+   * AdwComboRow:expression: (type GtkExpression)
    *
    * An expression used to obtain strings from items.
    *
@@ -571,7 +662,7 @@ adw_combo_row_class_init (AdwComboRowClass *klass)
                                G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwComboRow:use-subtitle: (attributes org.gtk.Property.get=adw_combo_row_get_use_subtitle org.gtk.Property.set=adw_combo_row_set_use_subtitle)
+   * AdwComboRow:use-subtitle:
    *
    * Whether to use the current value as the subtitle.
    *
@@ -589,7 +680,7 @@ adw_combo_row_class_init (AdwComboRowClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwComboRow:enable-search: (attributes org.gtk.Property.get=adw_combo_row_get_enable_search org.gtk.Property.set=adw_combo_row_set_enable_search)
+   * AdwComboRow:enable-search:
    *
    * Whether to show a search entry in the popup.
    *
@@ -605,6 +696,19 @@ adw_combo_row_class_init (AdwComboRowClass *klass)
                           FALSE,
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
+  /**
+   * AdwComboRow:search-match-mode:
+   *
+   * The match mode for the search filter.
+   *
+   * Since: 1.6
+   */
+  props[PROP_SEARCH_MATCH_MODE] =
+    g_param_spec_enum  ("search-match-mode", NULL, NULL,
+                        GTK_TYPE_STRING_FILTER_MATCH_MODE,
+                        GTK_STRING_FILTER_MATCH_MODE_PREFIX,
+                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
   g_object_class_install_properties (object_class, LAST_PROP, props);
 
   gtk_widget_class_set_template_from_resource (widget_class,
@@ -625,6 +729,10 @@ adw_combo_row_class_init (AdwComboRowClass *klass)
 static void
 adw_combo_row_init (AdwComboRow *self)
 {
+  AdwComboRowPrivate *priv = adw_combo_row_get_instance_private (self);
+
+  priv->search_match_mode = GTK_STRING_FILTER_MATCH_MODE_PREFIX;
+
   gtk_widget_init_template (GTK_WIDGET (self));
 
   adw_preferences_row_set_use_markup (ADW_PREFERENCES_ROW (self), FALSE);
@@ -646,7 +754,7 @@ adw_combo_row_new (void)
 }
 
 /**
- * adw_combo_row_set_selected: (attributes org.gtk.Method.set_property=selected)
+ * adw_combo_row_set_selected:
  * @self: a combo row
  * @position: the position of the item to select, or
  *   [const@Gtk.INVALID_LIST_POSITION]
@@ -673,7 +781,7 @@ adw_combo_row_set_selected (AdwComboRow *self,
 }
 
 /**
- * adw_combo_row_get_selected: (attributes org.gtk.Method.get_property=selected)
+ * adw_combo_row_get_selected:
  * @self: a combo row
  *
  * Gets the position of the selected item.
@@ -697,7 +805,7 @@ adw_combo_row_get_selected (AdwComboRow *self)
 }
 
 /**
- * adw_combo_row_get_selected_item: (attributes org.gtk.Method.get_property=selected-item)
+ * adw_combo_row_get_selected_item:
  * @self: a combo row
  *
  * Gets the selected item.
@@ -720,7 +828,7 @@ adw_combo_row_get_selected_item (AdwComboRow *self)
 }
 
 /**
- * adw_combo_row_get_model: (attributes org.gtk.Method.get_property=model)
+ * adw_combo_row_get_model:
  * @self: a combo row
  *
  * Gets the model that provides the displayed items.
@@ -740,7 +848,7 @@ adw_combo_row_get_model (AdwComboRow *self)
 }
 
 /**
- * adw_combo_row_set_model: (attributes org.gtk.Method.set_property=model)
+ * adw_combo_row_set_model:
  * @self: a combo row
  * @model: (nullable) (transfer none): the model to use
  *
@@ -766,6 +874,7 @@ adw_combo_row_set_model (AdwComboRow *self,
 
     if (priv->selection) {
       g_signal_handlers_disconnect_by_func (priv->selection, selection_changed, self);
+      g_signal_handlers_disconnect_by_func (priv->selection, selection_item_changed, self);
       g_signal_handlers_disconnect_by_func (priv->selection, model_changed, self);
     }
 
@@ -787,7 +896,7 @@ adw_combo_row_set_model (AdwComboRow *self,
     gtk_list_view_set_model (priv->list, selection);
     g_object_unref (selection);
 
-    selection = GTK_SELECTION_MODEL (gtk_single_selection_new (g_object_ref (filter_model)));
+    selection = GTK_SELECTION_MODEL (gtk_single_selection_new (g_object_ref (model)));
     g_set_object (&priv->selection, selection);
     g_object_unref (selection);
 
@@ -800,9 +909,11 @@ adw_combo_row_set_model (AdwComboRow *self,
     g_object_unref (selection);
 
     g_signal_connect_swapped (priv->selection, "notify::selected", G_CALLBACK (selection_changed), self);
+    g_signal_connect_swapped (priv->selection, "notify::selected-item", G_CALLBACK (selection_item_changed), self);
     g_signal_connect_swapped (priv->selection, "items-changed", G_CALLBACK (model_changed), self);
 
     selection_changed (self);
+    selection_item_changed (self);
     model_changed (self);
   }
 
@@ -810,7 +921,7 @@ adw_combo_row_set_model (AdwComboRow *self,
 }
 
 /**
- * adw_combo_row_get_factory: (attributes org.gtk.Method.get_property=factory)
+ * adw_combo_row_get_factory:
  * @self: a combo row
  *
  * Gets the factory for populating list items.
@@ -830,7 +941,7 @@ adw_combo_row_get_factory (AdwComboRow *self)
 }
 
 /**
- * adw_combo_row_set_factory: (attributes org.gtk.Method.set_property=factory)
+ * adw_combo_row_set_factory:
  * @self: a combo row
  * @factory: (nullable) (transfer none): the factory to use
  *
@@ -842,6 +953,47 @@ adw_combo_row_get_factory (AdwComboRow *self)
 void
 adw_combo_row_set_factory (AdwComboRow        *self,
                            GtkListItemFactory *factory)
+{
+  g_return_if_fail (ADW_IS_COMBO_ROW (self));
+  g_return_if_fail (factory == NULL || GTK_LIST_ITEM_FACTORY (factory));
+
+  set_factory (self, factory, FALSE);
+}
+
+/**
+ * adw_combo_row_get_header_factory:
+ * @self: a combo row
+ *
+ * Gets the factory that's currently used to create header widgets for the popup.
+ *
+ * Returns: (nullable) (transfer none): The factory in use
+ *
+ * Since: 1.6
+ */
+GtkListItemFactory *
+adw_combo_row_get_header_factory (AdwComboRow *self)
+{
+  AdwComboRowPrivate *priv;
+
+  g_return_val_if_fail (ADW_IS_COMBO_ROW (self), NULL);
+
+  priv = adw_combo_row_get_instance_private (self);
+
+  return priv->header_factory;
+}
+
+/**
+ * adw_combo_row_set_header_factory:
+ * @self: a combo row
+ * @factory: (nullable) (transfer none): the factory to use
+ *
+ * Sets the factory to use for creating header widgets for the popup.
+ *
+ * Since: 1.6
+ */
+void
+adw_combo_row_set_header_factory (AdwComboRow        *self,
+                                  GtkListItemFactory *factory)
 {
   AdwComboRowPrivate *priv;
 
@@ -850,19 +1002,16 @@ adw_combo_row_set_factory (AdwComboRow        *self,
 
   priv = adw_combo_row_get_instance_private (self);
 
-  if (!g_set_object (&priv->factory, factory))
+  if (!g_set_object (&priv->header_factory, factory))
     return;
 
-  gtk_list_view_set_factory (priv->current, factory);
+  gtk_list_view_set_header_factory (GTK_LIST_VIEW (priv->list), priv->header_factory);
 
-  if (priv->list_factory == NULL)
-    gtk_list_view_set_factory (priv->list, factory);
-
-  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_FACTORY]);
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_HEADER_FACTORY]);
 }
 
 /**
- * adw_combo_row_get_list_factory: (attributes org.gtk.Method.get_property=list-factory)
+ * adw_combo_row_get_list_factory:
  * @self: a combo row
  *
  * Gets the factory for populating list items in the popup.
@@ -882,7 +1031,7 @@ adw_combo_row_get_list_factory (AdwComboRow *self)
 }
 
 /**
- * adw_combo_row_set_list_factory: (attributes org.gtk.Method.set_property=list-factory)
+ * adw_combo_row_set_list_factory:
  * @self: a combo row
  * @factory: (nullable) (transfer none): the factory to use
  *
@@ -913,7 +1062,7 @@ adw_combo_row_set_list_factory (AdwComboRow        *self,
 }
 
 /**
- * adw_combo_row_get_expression: (attributes org.gtk.Method.get_property=expression)
+ * adw_combo_row_get_expression:
  * @self: a combo row
  *
  * Gets the expression used to obtain strings from items.
@@ -933,7 +1082,7 @@ adw_combo_row_get_expression (AdwComboRow *self)
 }
 
 /**
- * adw_combo_row_set_expression: (attributes org.gtk.Method.set_property=expression)
+ * adw_combo_row_set_expression:
  * @self: a combo row
  * @expression: (nullable): an expression
  *
@@ -970,11 +1119,14 @@ adw_combo_row_set_expression (AdwComboRow   *self,
 
   update_filter (self);
 
+  if (priv->uses_default_factory)
+    set_default_factory (self);
+
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_EXPRESSION]);
 }
 
 /**
- * adw_combo_row_get_use_subtitle: (attributes org.gtk.Method.get_property=use-subtitle)
+ * adw_combo_row_get_use_subtitle:
  * @self: a combo row
  *
  * Gets whether to use the current value as the subtitle.
@@ -994,7 +1146,7 @@ adw_combo_row_get_use_subtitle (AdwComboRow *self)
 }
 
 /**
- * adw_combo_row_set_use_subtitle: (attributes org.gtk.Method.set_property=use-subtitle)
+ * adw_combo_row_set_use_subtitle:
  * @self: a combo row
  * @use_subtitle: whether to use the current value as the subtitle
  *
@@ -1032,7 +1184,7 @@ adw_combo_row_set_use_subtitle (AdwComboRow *self,
 }
 
 /**
- * adw_combo_row_get_enable_search: (attributes org.gtk.Method.set_property=enable-search)
+ * adw_combo_row_get_enable_search:
  * @self: a combo row
  *
  * Gets whether search is enabled.
@@ -1059,7 +1211,7 @@ adw_combo_row_get_enable_search (AdwComboRow *self)
 }
 
 /**
- * adw_combo_row_set_enable_search: (attributes org.gtk.Method.set_property=enable-search)
+ * adw_combo_row_set_enable_search:
  * @self: a combo row
  * @enable_search: whether to enable search
  *
@@ -1094,3 +1246,54 @@ adw_combo_row_set_enable_search (AdwComboRow *self,
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_USE_SUBTITLE]);
 }
+
+/**
+ * adw_combo_row_get_search_match_mode:
+ * @self: a combo row
+ *
+ * Returns the match mode that the search filter is using.
+ *
+ * Returns: the match mode of the search filter
+ *
+ * Since: 1.6
+ */
+GtkStringFilterMatchMode
+adw_combo_row_get_search_match_mode (AdwComboRow *self)
+{
+  AdwComboRowPrivate *priv;
+
+  g_return_val_if_fail (ADW_IS_COMBO_ROW (self), GTK_STRING_FILTER_MATCH_MODE_PREFIX);
+
+  priv = adw_combo_row_get_instance_private (self);
+
+  return priv->search_match_mode;
+}
+
+/**
+ * adw_combo_row_set_search_match_mode:
+ * @self: a combo row
+ * @search_match_mode: the new match mode
+ *
+ * Sets the match mode for the search filter.
+ *
+ * Since: 1.6
+ */
+void
+adw_combo_row_set_search_match_mode (AdwComboRow *self,
+                                     GtkStringFilterMatchMode search_match_mode)
+{
+  AdwComboRowPrivate *priv;
+
+  g_return_if_fail (ADW_IS_COMBO_ROW (self));
+
+  priv = adw_combo_row_get_instance_private (self);
+
+  if (priv->search_match_mode == search_match_mode)
+    return;
+
+  priv->search_match_mode = search_match_mode;
+
+  update_filter (self);
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_SEARCH_MATCH_MODE]);
+}
