@@ -25,11 +25,15 @@
 
 #include "adw-back-button-private.h"
 #include "adw-bin.h"
+#include "adw-bottom-sheet-private.h"
+#include "adw-dialog.h"
 #include "adw-enums.h"
+#include "adw-floating-sheet-private.h"
 #include "adw-gizmo-private.h"
 #include "adw-navigation-split-view.h"
 #include "adw-navigation-view.h"
 #include "adw-overlay-split-view.h"
+#include "adw-sheet-controls-private.h"
 #include "adw-widget-utils-private.h"
 
 /**
@@ -46,6 +50,15 @@
  * features compared to it. Refer to `GtkHeaderBar` for details. It is typically
  * used as a top bar within [class@ToolbarView].
  *
+ * ## Dialog Integration
+ *
+ * When placed inside an [class@Dialog], `AdwHeaderBar` will display the dialog
+ * title intead of window title. It will also adjust the decoration layout to
+ * ensure it always has a close button and nothing else. Set
+ * [property@HeaderBar:show-start-title-buttons] and
+ * [property@HeaderBar:show-end-title-buttons] to `FALSE` to remove it if it's
+ * unwanted.
+ *
  * ## Navigation View Integration
  *
  * When placed inside an [class@NavigationPage], `AdwHeaderBar` will display the
@@ -54,16 +67,24 @@
  * When used together with [class@NavigationView] or [class@NavigationSplitView],
  * it will also display a back button that can be used to go back to the previous
  * page. The button also has a context menu, allowing to pop multiple pages at
- * once, potentially across multiple navigation views. Set
- * [property@HeaderBar:show-back-button] to `FALSE` to disable the back button
- * if it's unwanted.
+ * once, potentially across multiple navigation views.
+ *
+ * Set [property@HeaderBar:show-back-button] to `FALSE` to disable this behavior
+ * in rare scenarios where it's unwanted.
  *
  * ## Split View Integration
  *
- * When placed inside `AdwNavigationSplitView` or `AdwOverlaySplitView`,
+ * When placed inside [class@NavigationSplitView] or [class@OverlaySplitView],
  * `AdwHeaderBar` will automatically hide the title buttons other than at the
  * edges of the window.
  *
+ * ## Bottom Sheet Integration
+ *
+ * When played inside [class@BottomSheet], `AdwHeaderBar` will not show the title
+ * unless [property@BottomSheet:show-drag-handle] is set to `FALSE`, regardless
+ * of [property@HeaderBar:show-title]. This only applies to the default title,
+ * titles set with [property@HeaderBar:title-widget] will still be shown.
+ *
  * ## Centering Policy
  *
  * [property@HeaderBar:centering-policy] allows to enforce strict centering of
@@ -147,8 +168,8 @@ struct _AdwHeaderBar {
   GtkWidget *title_label;
   GtkWidget *title_widget;
 
-  GtkWidget *start_window_controls;
-  GtkWidget *end_window_controls;
+  GtkWidget *start_controls;
+  GtkWidget *end_controls;
   GtkWidget *back_button;
 
   char *decoration_layout;
@@ -159,11 +180,12 @@ struct _AdwHeaderBar {
   guint track_default_decoration : 1;
 
   AdwCenteringPolicy centering_policy;
-  gboolean is_mobile_window;
 
   GtkSizeGroup *size_group;
 
   GtkWidget *title_navigation_page;
+  GtkWidget *dialog;
+  GtkWidget *sheet;
 
   GSList *split_views;
 };
@@ -190,31 +212,105 @@ G_DEFINE_FINAL_TYPE_WITH_CODE (AdwHeaderBar, adw_header_bar, GTK_TYPE_WIDGET,
 static GtkBuildableIface *parent_buildable_iface;
 
 static void
-create_start_window_controls (AdwHeaderBar *self)
+update_box_visibility (GtkWidget *box)
 {
-  GtkWidget *controls = gtk_window_controls_new (GTK_PACK_START);
-  g_object_bind_property (self, "decoration-layout",
-                          controls, "decoration-layout",
-                          G_BINDING_SYNC_CREATE);
-  g_object_bind_property (controls, "empty",
-                          controls, "visible",
-                          G_BINDING_SYNC_CREATE | G_BINDING_INVERT_BOOLEAN);
-  gtk_box_prepend (GTK_BOX (self->start_box), controls);
-  self->start_window_controls = controls;
+  gboolean has_visible = FALSE;
+  GtkWidget *child;
+
+  for (child = gtk_widget_get_first_child (box);
+       child;
+       child = gtk_widget_get_next_sibling (child)) {
+    if (gtk_widget_get_visible (child)) {
+      has_visible = TRUE;
+      break;
+    }
+  }
+
+  gtk_widget_set_visible (box, has_visible);
 }
 
 static void
-create_end_window_controls (AdwHeaderBar *self)
+update_decoration_layout (AdwHeaderBar *self,
+                          gboolean      start,
+                          gboolean      end)
 {
-  GtkWidget *controls = gtk_window_controls_new (GTK_PACK_END);
-  g_object_bind_property (self, "decoration-layout",
-                          controls, "decoration-layout",
-                          G_BINDING_SYNC_CREATE);
+  if (start && self->start_controls) {
+    g_object_set (self->start_controls,
+                  "decoration-layout", self->decoration_layout,
+                  NULL);
+  }
+
+  if (end && self->end_controls) {
+    g_object_set (self->end_controls,
+                  "decoration-layout", self->decoration_layout,
+                  NULL);
+  }
+}
+
+static void
+recreate_start_controls (AdwHeaderBar *self)
+{
+  GtkWidget *controls;
+
+  if (self->start_controls) {
+    if (self->sheet && ADW_IS_SHEET_CONTROLS (self->start_controls))
+      return;
+
+    if (!self->sheet && GTK_IS_WINDOW_CONTROLS (self->start_controls))
+      return;
+
+    gtk_box_remove (GTK_BOX (self->start_box), self->start_controls);
+    self->start_controls = NULL;
+  }
+
+  if (self->sheet)
+    controls = adw_sheet_controls_new (GTK_PACK_START);
+  else
+    controls = gtk_window_controls_new (GTK_PACK_START);
+
   g_object_bind_property (controls, "empty",
                           controls, "visible",
                           G_BINDING_SYNC_CREATE | G_BINDING_INVERT_BOOLEAN);
+  g_signal_connect_swapped (controls, "notify::visible",
+                            G_CALLBACK (update_box_visibility),
+                            self->start_box);
+  gtk_box_prepend (GTK_BOX (self->start_box), controls);
+  self->start_controls = controls;
+
+  update_decoration_layout (self, TRUE, FALSE);
+}
+
+static void
+recreate_end_controls (AdwHeaderBar *self)
+{
+  GtkWidget *controls;
+
+  if (self->end_controls) {
+    if (self->sheet && ADW_IS_SHEET_CONTROLS (self->end_controls))
+      return;
+
+    if (!self->sheet && GTK_IS_WINDOW_CONTROLS (self->end_controls))
+      return;
+
+    gtk_box_remove (GTK_BOX (self->end_box), self->end_controls);
+    self->end_controls = NULL;
+  }
+
+  if (self->sheet)
+    controls = adw_sheet_controls_new (GTK_PACK_END);
+  else
+    controls = gtk_window_controls_new (GTK_PACK_END);
+
+  g_object_bind_property (controls, "empty",
+                          controls, "visible",
+                          G_BINDING_SYNC_CREATE | G_BINDING_INVERT_BOOLEAN);
+  g_signal_connect_swapped (controls, "notify::visible",
+                            G_CALLBACK (update_box_visibility),
+                            self->end_box);
   gtk_box_append (GTK_BOX (self->end_box), controls);
-  self->end_window_controls = controls;
+  self->end_controls = controls;
+
+  update_decoration_layout (self, FALSE, TRUE);
 }
 
 static void
@@ -224,7 +320,10 @@ create_back_button (AdwHeaderBar *self)
 
   gtk_box_insert_child_after (GTK_BOX (self->start_box),
                               button,
-                              self->start_window_controls);
+                              self->start_controls);
+  g_signal_connect_swapped (button, "notify::visible",
+                            G_CALLBACK (update_box_visibility),
+                            self->start_box);
 
   self->back_button = button;
 }
@@ -258,15 +357,14 @@ update_start_title_buttons (AdwHeaderBar *self)
     }
   }
 
-  if ((self->start_window_controls != NULL) == show)
-    return;
-
   if (show) {
-    create_start_window_controls (self);
-  } else if (self->start_box && self->start_window_controls) {
-    gtk_box_remove (GTK_BOX (self->start_box), self->start_window_controls);
-    self->start_window_controls = NULL;
+    recreate_start_controls (self);
+  } else if (self->start_box && self->start_controls) {
+    gtk_box_remove (GTK_BOX (self->start_box), self->start_controls);
+    self->start_controls = NULL;
   }
+
+  update_box_visibility (self->start_box);
 }
 
 static void
@@ -298,15 +396,14 @@ update_end_title_buttons (AdwHeaderBar *self)
     }
   }
 
-  if ((self->end_window_controls != NULL) == show)
-    return;
-
   if (show) {
-    create_end_window_controls (self);
-  } else if (self->end_box && self->end_window_controls) {
-    gtk_box_remove (GTK_BOX (self->end_box), self->end_window_controls);
-    self->end_window_controls = NULL;
+    recreate_end_controls (self);
+  } else if (self->end_box && self->end_controls) {
+    gtk_box_remove (GTK_BOX (self->end_box), self->end_controls);
+    self->end_controls = NULL;
   }
+
+  update_box_visibility (self->end_box);
 }
 
 static void
@@ -324,9 +421,18 @@ update_title (AdwHeaderBar *self)
   if (!self->title_label)
     return;
 
+  if (ADW_IS_BOTTOM_SHEET (self->sheet) &&
+      adw_bottom_sheet_get_show_drag_handle (ADW_BOTTOM_SHEET (self->sheet))) {
+    gtk_label_set_text (GTK_LABEL (self->title_label), NULL);
+    return;
+  }
+
   if (self->title_navigation_page)
     title = adw_navigation_page_get_title (ADW_NAVIGATION_PAGE (self->title_navigation_page));
 
+  if (!title && self->dialog)
+    title = adw_dialog_get_title (ADW_DIALOG (self->dialog));
+
   if (!title) {
     GtkRoot *root = gtk_widget_get_root (GTK_WIDGET (self));
 
@@ -365,6 +471,33 @@ construct_title_label (AdwHeaderBar *self)
   update_title (self);
 }
 
+static GtkWidget *
+find_sheet (GtkWidget *widget)
+{
+  GtkWidget *sheet, *bin, *parent;
+
+  sheet = adw_widget_get_ancestor (widget, ADW_TYPE_BOTTOM_SHEET, TRUE, FALSE);
+
+  if (!sheet)
+    sheet = adw_widget_get_ancestor (widget, ADW_TYPE_FLOATING_SHEET, TRUE, FALSE);
+
+  if (ADW_IS_BOTTOM_SHEET (sheet))
+    bin = adw_bottom_sheet_get_sheet_bin (ADW_BOTTOM_SHEET (sheet));
+  else if (ADW_IS_FLOATING_SHEET (sheet))
+    bin = adw_floating_sheet_get_sheet_bin (ADW_FLOATING_SHEET (sheet));
+  else
+    return NULL;
+
+  if (bin && (widget == bin || gtk_widget_is_ancestor (widget, bin)))
+    return sheet;
+
+  parent = gtk_widget_get_parent (sheet);
+  if (parent)
+    return find_sheet (parent);
+
+  return NULL;
+}
+
 static void
 adw_header_bar_root (GtkWidget *widget)
 {
@@ -374,11 +507,22 @@ adw_header_bar_root (GtkWidget *widget)
   GTK_WIDGET_CLASS (adw_header_bar_parent_class)->root (widget);
 
   self->title_navigation_page =
-    adw_widget_get_ancestor_same_native (widget, ADW_TYPE_NAVIGATION_PAGE);
+    adw_widget_get_ancestor (widget, ADW_TYPE_NAVIGATION_PAGE, TRUE, TRUE);
+
+  self->dialog = adw_widget_get_ancestor (widget, ADW_TYPE_DIALOG, TRUE, FALSE);
+  self->sheet = find_sheet (widget);
+
+  if (ADW_IS_BOTTOM_SHEET (self->sheet)) {
+    g_signal_connect_swapped (self->sheet, "notify::show-drag-handle",
+                              G_CALLBACK (update_title), widget);
+  }
 
   if (self->title_navigation_page) {
     g_signal_connect_swapped (self->title_navigation_page, "notify::title",
                               G_CALLBACK (update_title), widget);
+  } else if (self->dialog) {
+    g_signal_connect_swapped (self->dialog, "notify::title",
+                              G_CALLBACK (update_title), widget);
   } else {
     GtkRoot *root = gtk_widget_get_root (widget);
 
@@ -442,6 +586,7 @@ adw_header_bar_root (GtkWidget *widget)
 
   update_title (self);
   update_title_buttons (self);
+  update_decoration_layout (self, TRUE, TRUE);
 }
 
 static void
@@ -453,13 +598,23 @@ adw_header_bar_unroot (GtkWidget *widget)
   if (self->title_navigation_page) {
     g_signal_handlers_disconnect_by_func (self->title_navigation_page,
                                           update_title, widget);
-
-    self->title_navigation_page = NULL;
+  } else if (self->dialog) {
+    g_signal_handlers_disconnect_by_func (self->dialog,
+                                          update_title, widget);
   } else {
     g_signal_handlers_disconnect_by_func (gtk_widget_get_root (widget),
                                           update_title, widget);
   }
 
+  if (self->sheet) {
+    g_signal_handlers_disconnect_by_func (self->sheet,
+                                          update_title, widget);
+  }
+
+  self->title_navigation_page = NULL;
+  self->dialog = NULL;
+  self->sheet = FALSE;
+
   for (l = self->split_views; l; l = l->next) {
     SplitViewData *data = l->data;
 
@@ -593,7 +748,7 @@ adw_header_bar_class_init (AdwHeaderBarClass *class)
   widget_class->compute_expand = adw_widget_compute_expand_horizontal_only;
 
   /**
-   * AdwHeaderBar:title-widget: (attributes org.gtk.Property.get=adw_header_bar_get_title_widget org.gtk.Property.set=adw_header_bar_set_title_widget)
+   * AdwHeaderBar:title-widget:
    *
    * The title widget to display.
    *
@@ -618,7 +773,7 @@ adw_header_bar_class_init (AdwHeaderBarClass *class)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwHeaderBar:show-start-title-buttons: (attributes org.gtk.Property.get=adw_header_bar_get_show_start_title_buttons org.gtk.Property.set=adw_header_bar_set_show_start_title_buttons)
+   * AdwHeaderBar:show-start-title-buttons:
    *
    * Whether to show title buttons at the start of the header bar.
    *
@@ -635,7 +790,7 @@ adw_header_bar_class_init (AdwHeaderBarClass *class)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwHeaderBar:show-end-title-buttons: (attributes org.gtk.Property.get=adw_header_bar_get_show_end_title_buttons org.gtk.Property.set=adw_header_bar_set_show_end_title_buttons)
+   * AdwHeaderBar:show-end-title-buttons:
    *
    * Whether to show title buttons at the end of the header bar.
    *
@@ -652,12 +807,12 @@ adw_header_bar_class_init (AdwHeaderBarClass *class)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwHeaderBar:show-back-button: (attributes org.gtk.Property.get=adw_header_bar_get_show_back_button org.gtk.Property.set=adw_header_bar_set_show_back_button)
+   * AdwHeaderBar:show-back-button:
    *
    * Whether the header bar can show the back button.
    *
    * The back button will never be shown unless the header bar is placed inside an
-   * [class@NavigationView].
+   * [class@NavigationView]. Usually, there is no reason to set this to `FALSE`.
    *
    * Since: 1.4
    */
@@ -667,7 +822,7 @@ adw_header_bar_class_init (AdwHeaderBarClass *class)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwHeaderBar:decoration-layout: (attributes org.gtk.Property.get=adw_header_bar_get_decoration_layout org.gtk.Property.set=adw_header_bar_set_decoration_layout)
+   * AdwHeaderBar:decoration-layout:
    *
    * The decoration layout for buttons.
    *
@@ -688,7 +843,7 @@ adw_header_bar_class_init (AdwHeaderBarClass *class)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwHeaderBar:centering-policy: (attributes org.gtk.Property.get=adw_header_bar_get_centering_policy org.gtk.Property.set=adw_header_bar_set_centering_policy)
+   * AdwHeaderBar:centering-policy:
    *
    * The policy for aligning the center widget.
    */
@@ -699,7 +854,7 @@ adw_header_bar_class_init (AdwHeaderBarClass *class)
                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwHeaderBar:show-title: (attributes org.gtk.Property.get=adw_header_bar_get_show_title org.gtk.Property.set=adw_header_bar_set_show_title)
+   * AdwHeaderBar:show-title:
    *
    * Whether the title widget should be shown.
    *
@@ -730,7 +885,7 @@ adw_header_bar_init (AdwHeaderBar *self)
   gtk_widget_set_parent (self->handle, GTK_WIDGET (self));
 
   self->center_box = gtk_center_box_new ();
-  gtk_center_box_set_shrink_center_last (GTK_CENTER_BOX (self->center_box), TRUE);
+  gtk_center_box_set_shrink_center_last (GTK_CENTER_BOX (self->center_box), FALSE);
   gtk_window_handle_set_child (GTK_WINDOW_HANDLE (self->handle), self->center_box);
 
   self->start_bin = adw_gizmo_new ("widget", NULL, NULL, NULL, NULL,
@@ -761,8 +916,6 @@ adw_header_bar_init (AdwHeaderBar *self)
   self->size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 
   construct_title_label (self);
-  create_start_window_controls (self);
-  create_end_window_controls (self);
   create_back_button (self);
 }
 
@@ -816,7 +969,16 @@ void
 adw_header_bar_pack_start (AdwHeaderBar *self,
                            GtkWidget    *child)
 {
+  g_return_if_fail (ADW_IS_HEADER_BAR (self));
+  g_return_if_fail (GTK_IS_WIDGET (child));
+  g_return_if_fail (gtk_widget_get_parent (child) == NULL);
+
   gtk_box_append (GTK_BOX (self->start_box), child);
+  update_box_visibility (self->start_box);
+
+  g_signal_connect_swapped (child, "notify::visible",
+                            G_CALLBACK (update_box_visibility),
+                            self->start_box);
 }
 
 /**
@@ -830,7 +992,16 @@ void
 adw_header_bar_pack_end (AdwHeaderBar *self,
                          GtkWidget    *child)
 {
+  g_return_if_fail (ADW_IS_HEADER_BAR (self));
+  g_return_if_fail (GTK_IS_WIDGET (child));
+  g_return_if_fail (gtk_widget_get_parent (child) == NULL);
+
   gtk_box_prepend (GTK_BOX (self->end_box), child);
+  update_box_visibility (self->end_box);
+
+  g_signal_connect_swapped (child, "notify::visible",
+                            G_CALLBACK (update_box_visibility),
+                            self->end_box);
 }
 
 /**
@@ -854,18 +1025,23 @@ adw_header_bar_remove (AdwHeaderBar *self,
 
   parent = gtk_widget_get_parent (child);
 
-  if (parent == self->start_box)
-    gtk_box_remove (GTK_BOX (self->start_box), child);
-  else if (parent == self->end_box)
-    gtk_box_remove (GTK_BOX (self->end_box), child);
-  else if (parent == self->center_bin)
+  if (parent == self->start_box || parent == self->end_box) {
+    g_signal_handlers_disconnect_by_func (child,
+                                          update_box_visibility,
+                                          parent);
+
+    gtk_box_remove (GTK_BOX (parent), child);
+
+    update_box_visibility (parent);
+  } else if (parent == self->center_bin) {
     adw_bin_set_child (ADW_BIN (self->center_bin), NULL);
-  else
+  } else {
     ADW_CRITICAL_CANNOT_REMOVE_CHILD (self, child);
+  }
 }
 
 /**
- * adw_header_bar_get_title_widget: (attributes org.gtk.Method.get_property=title-widget)
+ * adw_header_bar_get_title_widget:
  * @self: a header bar
  *
  * Gets the title widget widget of @self.
@@ -881,7 +1057,7 @@ adw_header_bar_get_title_widget (AdwHeaderBar *self)
 }
 
 /**
- * adw_header_bar_set_title_widget: (attributes org.gtk.Method.set_property=title-widget)
+ * adw_header_bar_set_title_widget:
  * @self: a header bar
  * @title_widget: (nullable): a widget to use for a title
  *
@@ -931,7 +1107,7 @@ adw_header_bar_set_title_widget (AdwHeaderBar *self,
 }
 
 /**
- * adw_header_bar_get_show_start_title_buttons: (attributes org.gtk.Method.get_property=show-start-title-buttons)
+ * adw_header_bar_get_show_start_title_buttons:
  * @self: a header bar
  *
  * Gets whether to show title buttons at the start of @self.
@@ -947,7 +1123,7 @@ adw_header_bar_get_show_start_title_buttons (AdwHeaderBar *self)
 }
 
 /**
- * adw_header_bar_set_show_start_title_buttons: (attributes org.gtk.Method.set_property=show-start-title-buttons)
+ * adw_header_bar_set_show_start_title_buttons:
  * @self: a header bar
  * @setting: `TRUE` to show standard title buttons
  *
@@ -979,7 +1155,7 @@ adw_header_bar_set_show_start_title_buttons (AdwHeaderBar *self,
 }
 
 /**
- * adw_header_bar_get_show_end_title_buttons: (attributes org.gtk.Method.get_property=show-end-title-buttons)
+ * adw_header_bar_get_show_end_title_buttons:
  * @self: a header bar
  *
  * Gets whether to show title buttons at the end of @self.
@@ -995,7 +1171,7 @@ adw_header_bar_get_show_end_title_buttons (AdwHeaderBar *self)
 }
 
 /**
- * adw_header_bar_set_show_end_title_buttons: (attributes org.gtk.Method.set_property=show-end-title-buttons)
+ * adw_header_bar_set_show_end_title_buttons:
  * @self: a header bar
  * @setting: `TRUE` to show standard title buttons
  *
@@ -1027,7 +1203,7 @@ adw_header_bar_set_show_end_title_buttons (AdwHeaderBar *self,
 }
 
 /**
- * adw_header_bar_get_show_back_button: (attributes org.gtk.Method.get_property=show-back-button)
+ * adw_header_bar_get_show_back_button:
  * @self: a header bar
  *
  * Gets whether @self can show the back button.
@@ -1045,14 +1221,14 @@ adw_header_bar_get_show_back_button (AdwHeaderBar *self)
 }
 
 /**
- * adw_header_bar_set_show_back_button: (attributes org.gtk.Method.set_property=show-back-button)
+ * adw_header_bar_set_show_back_button:
  * @self: a header bar
  * @show_back_button: whether to show the back button
  *
  * Sets whether @self can show the back button.
  *
  * The back button will never be shown unless the header bar is placed inside an
- * [class@NavigationView].
+ * [class@NavigationView]. Usually, there is no reason to set it to `FALSE`.
  *
  * Since: 1.4
  */
@@ -1076,13 +1252,15 @@ adw_header_bar_set_show_back_button (AdwHeaderBar *self,
       gtk_box_remove (GTK_BOX (self->start_box), self->back_button);
       self->back_button = NULL;
     }
+
+    update_box_visibility (self->start_box);
   }
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_SHOW_BACK_BUTTON]);
 }
 
 /**
- * adw_header_bar_get_decoration_layout: (attributes org.gtk.Method.get_property=decoration-layout)
+ * adw_header_bar_get_decoration_layout:
  * @self: a header bar
  *
  * Gets the decoration layout for @self.
@@ -1098,7 +1276,7 @@ adw_header_bar_get_decoration_layout (AdwHeaderBar *self)
 }
 
 /**
- * adw_header_bar_set_decoration_layout: (attributes org.gtk.Method.set_property=decoration-layout)
+ * adw_header_bar_set_decoration_layout:
  * @self: a header bar
  * @layout: (nullable): a decoration layout
  *
@@ -1121,14 +1299,16 @@ adw_header_bar_set_decoration_layout (AdwHeaderBar *self,
 {
   g_return_if_fail (ADW_IS_HEADER_BAR (self));
 
-  g_clear_pointer (&self->decoration_layout, g_free);
-  self->decoration_layout = g_strdup (layout);
+  if (!g_set_str (&self->decoration_layout, layout))
+    return;
+
+  update_decoration_layout (self, TRUE, TRUE);
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_DECORATION_LAYOUT]);
 }
 
 /**
- * adw_header_bar_get_centering_policy: (attributes org.gtk.Method.get_property=centering-policy)
+ * adw_header_bar_get_centering_policy:
  * @self: a header bar
  *
  * Gets the policy for aligning the center widget.
@@ -1144,7 +1324,7 @@ adw_header_bar_get_centering_policy (AdwHeaderBar *self)
 }
 
 /**
- * adw_header_bar_set_centering_policy: (attributes org.gtk.Method.set_property=centering-policy)
+ * adw_header_bar_set_centering_policy:
  * @self: a header bar
  * @centering_policy: the centering policy
  *
@@ -1173,7 +1353,7 @@ adw_header_bar_set_centering_policy (AdwHeaderBar       *self,
 }
 
 /**
- * adw_header_bar_get_show_title: (attributes org.gtk.Method.get_property=show-title)
+ * adw_header_bar_get_show_title:
  * @self: a header bar
  *
  * Gets whether the title widget should be shown.
@@ -1191,7 +1371,7 @@ adw_header_bar_get_show_title (AdwHeaderBar *self)
 }
 
 /**
- * adw_header_bar_set_show_title: (attributes org.gtk.Method.set_property=show-title)
+ * adw_header_bar_set_show_title:
  * @self: a header bar
  * @show_title: whether the title widget is visible
  *
