@@ -9,6 +9,7 @@
 #include "config.h"
 #include "adw-message-dialog.h"
 
+#include "adw-bin.h"
 #include "adw-gizmo-private.h"
 #include "adw-gtkbuilder-utils-private.h"
 #include "adw-marshalers.h"
@@ -72,8 +73,7 @@
  * ## Async API
  *
  * `AdwMessageDialog` can also be used via the [method@MessageDialog.choose]
- * method. This API follows the GIO async pattern, and the result can be
- * obtained by calling [method@MessageDialog.choose_finish], for example:
+ * method. This API follows the GIO async pattern, for example:
  *
  * ```c
  * static void
@@ -149,27 +149,18 @@
  * `AdwMessageDialog` uses the `GTK_ACCESSIBLE_ROLE_DIALOG` role.
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 
-/**
- * AdwResponseAppearance:
- * @ADW_RESPONSE_DEFAULT: the default appearance.
- * @ADW_RESPONSE_SUGGESTED: used to denote important responses such as the
- *     affirmative action.
- * @ADW_RESPONSE_DESTRUCTIVE: used to draw attention to the potentially damaging
- *     consequences of using the response. This appearance acts as a warning to
- *     the user.
- *
- * Describes the possible styles of [class@MessageDialog] response buttons.
- *
- * See [method@MessageDialog.set_response_appearance].
- *
- * Since: 1.2
- */
-
-#define DIALOG_MARGIN 30
-#define DIALOG_MAX_WIDTH 550
+#define DIALOG_MARGIN_VERT 20
+#define DIALOG_MARGIN_HORZ 30
+#define DIALOG_MAX_WIDTH 372 /* sp, not px */
+#define DIALOG_MAX_WIDE_WIDTH 600 /* sp, not px */
+#define DIALOG_PREFERRED_WIDTH 300 /* sp, not px */
 #define DIALOG_MIN_WIDTH 300
+#define BUTTON_SPACING 12
+#define RESPONSE_HORZ_PADDING 48
+#define RESPONSE_HORZ_PADDING_SHORT 36
 
 typedef struct {
   AdwMessageDialog *dialog;
@@ -179,13 +170,15 @@ typedef struct {
   gboolean enabled;
 
   GtkWidget *button;
-  GtkWidget *separator;
 } ResponseInfo;
 
 typedef struct
 {
+  GtkWidget *heading_bin;
   GtkWidget *heading_label;
+  GtkWidget *heading_label_small;
   GtkWidget *body_label;
+  GtkWidget *child_bin;
   GtkBox *message_area;
   GtkWidget *response_area;
 
@@ -265,10 +258,21 @@ parent_size_cb (AdwMessageDialog *self)
 
   priv->parent_width = w;
   priv->parent_height = h;
+
+  if (priv->parent_width < 450)
+    gtk_widget_add_css_class (GTK_WIDGET (self), "narrow");
+  else
+    gtk_widget_remove_css_class (GTK_WIDGET (self), "narrow");
+
+  if (priv->parent_height < 360)
+    gtk_widget_add_css_class (GTK_WIDGET (self), "short");
+  else
+    gtk_widget_remove_css_class (GTK_WIDGET (self), "short");
+
   gtk_widget_queue_resize (GTK_WIDGET (self));
 }
 
-static gboolean
+static void
 parent_state_idle_cb (AdwMessageDialog *self)
 {
   AdwMessageDialogPrivate *priv = adw_message_dialog_get_instance_private (self);
@@ -276,8 +280,6 @@ parent_state_idle_cb (AdwMessageDialog *self)
   parent_size_cb (self);
 
   priv->parent_state_idle_id = 0;
-
-  return G_SOURCE_REMOVE;
 }
 
 static void
@@ -287,7 +289,8 @@ parent_state_cb (AdwMessageDialog *self)
 
   g_clear_handle_id (&priv->parent_state_idle_id, g_source_remove);
 
-  priv->parent_state_idle_id = g_idle_add (G_SOURCE_FUNC (parent_state_idle_cb), self);
+  priv->parent_state_idle_id =
+    g_idle_add_once ((GSourceOnceFunc) parent_state_idle_cb, self);
 }
 
 static void
@@ -331,6 +334,9 @@ parent_unrealize_cb (AdwMessageDialog *self)
 
   priv->parent_width = -1;
   priv->parent_height = -1;
+
+  gtk_widget_remove_css_class (GTK_WIDGET (self), "short");
+  gtk_widget_remove_css_class (GTK_WIDGET (self), "narrow");
 }
 
 static void
@@ -343,6 +349,9 @@ parent_window_notify_cb (AdwMessageDialog *self)
   priv->parent_window = NULL;
   priv->parent_width = -1;
   priv->parent_height = -1;
+
+  gtk_widget_remove_css_class (GTK_WIDGET (self), "short");
+  gtk_widget_remove_css_class (GTK_WIDGET (self), "narrow");
 }
 
 static void
@@ -419,15 +428,14 @@ create_response_button (AdwMessageDialog *self,
 {
   GtkWidget *button = gtk_button_new_with_mnemonic (info->label);
 
-  gtk_widget_add_css_class (button, "flat");
   gtk_button_set_can_shrink (GTK_BUTTON (button), TRUE);
 
   switch (info->appearance) {
   case ADW_RESPONSE_SUGGESTED:
-    gtk_widget_add_css_class (button, "suggested");
+    gtk_widget_add_css_class (button, "suggested-action");
     break;
   case ADW_RESPONSE_DESTRUCTIVE:
-    gtk_widget_add_css_class (button, "destructive");
+    gtk_widget_add_css_class (button, "destructive-action");
     break;
   case ADW_RESPONSE_DEFAULT:
   default:
@@ -530,6 +538,105 @@ adw_message_dialog_map (GtkWidget *widget)
   }
 }
 
+static GtkSizeRequestMode
+get_heading_request_mode (GtkWidget *widget)
+{
+  return GTK_SIZE_REQUEST_HEIGHT_FOR_WIDTH;
+}
+
+static void
+measure_heading (GtkWidget      *widget,
+                 GtkOrientation  orientation,
+                 int             for_size,
+                 int            *minimum,
+                 int            *natural,
+                 int            *minimum_baseline,
+                 int            *natural_baseline)
+{
+  AdwMessageDialog *self = ADW_MESSAGE_DIALOG (gtk_widget_get_root (widget));
+  AdwMessageDialogPrivate *priv = adw_message_dialog_get_instance_private (self);
+  int large_min, large_nat, small_min, small_nat;
+
+  if (gtk_widget_has_css_class (GTK_WIDGET (self), "short")) {
+    gtk_widget_measure (priv->heading_label_small, orientation, for_size,
+                        minimum, natural, NULL, NULL);
+
+    if (minimum_baseline)
+      *minimum_baseline = -1;
+    if (natural_baseline)
+      *natural_baseline = -1;
+
+    return;
+  }
+
+  gtk_widget_measure (priv->heading_label, orientation, for_size,
+                      &large_min, &large_nat, NULL, NULL);
+  gtk_widget_measure (priv->heading_label_small, orientation, for_size,
+                      &small_min, &small_nat, NULL, NULL);
+
+  if (orientation == GTK_ORIENTATION_HORIZONTAL) {
+    if (minimum)
+      *minimum = MIN (large_min, small_min);
+    if (natural)
+      *natural = MAX (large_nat, small_nat);
+  } else {
+    if (for_size < 0) {
+      if (minimum)
+        *minimum = MAX (large_min, small_min);
+      if (natural)
+        *natural = MAX (large_nat, small_nat);
+    } else {
+      int large_width;
+
+      gtk_widget_measure (priv->heading_label, GTK_ORIENTATION_HORIZONTAL, -1,
+                          NULL, &large_width, NULL, NULL);
+
+      if (minimum)
+        *minimum = (large_width > for_size) ? small_min : large_min;
+      if (natural)
+        *natural = (large_width > for_size) ? small_nat : large_nat;
+    }
+  }
+
+  if (minimum_baseline)
+    *minimum_baseline = -1;
+  if (natural_baseline)
+    *natural_baseline = -1;
+}
+
+static void
+allocate_heading (GtkWidget *widget,
+                  int        width,
+                  int        height,
+                  int        baseline)
+{
+  AdwMessageDialog *self = ADW_MESSAGE_DIALOG (gtk_widget_get_root (widget));
+  AdwMessageDialogPrivate *priv = adw_message_dialog_get_instance_private (self);
+  gboolean small_label;
+
+  if (gtk_widget_has_css_class (GTK_WIDGET (self), "short")) {
+    small_label = TRUE;
+  } else {
+    int large_nat;
+
+    gtk_widget_measure (priv->heading_label, GTK_ORIENTATION_HORIZONTAL, -1,
+                        NULL, &large_nat, NULL, NULL);
+
+    small_label = large_nat > width;
+  }
+
+  if (gtk_widget_get_child_visible (priv->heading_label) == small_label)
+    gtk_widget_set_child_visible (priv->heading_label, !small_label);
+
+  if (gtk_widget_get_child_visible (priv->heading_label_small) != small_label)
+    gtk_widget_set_child_visible (priv->heading_label_small, small_label);
+
+  if (small_label)
+    gtk_widget_allocate (priv->heading_label_small, width, height, baseline, NULL);
+  else
+    gtk_widget_allocate (priv->heading_label, width, height, baseline, NULL);
+}
+
 static void
 measure_responses_do (AdwMessageDialog *self,
                       gboolean          compact,
@@ -563,17 +670,9 @@ measure_responses_do (AdwMessageDialog *self,
       nat += child_nat;
     }
 
-    if (response->separator) {
-      gtk_widget_measure (response->separator, orientation, -1,
-                          &child_min, &child_nat, NULL, NULL);
-
-    if (horiz == compact) {
-        min = MAX (min, child_min);
-        nat = MAX (nat, child_nat);
-      } else {
-        min += child_min;
-        nat += child_nat;
-      }
+    if (horiz != compact && l->next) {
+      min += BUTTON_SPACING;
+      nat += BUTTON_SPACING;
     }
   }
 
@@ -610,11 +709,14 @@ measure_responses (GtkWidget      *widget,
     measure_responses_do (self, FALSE, orientation, NULL, natural);
   } else {
     int wide_nat = 0;
+    gboolean use_compact_layout;
 
     if (for_size >= 0)
       measure_responses_do (self, FALSE, GTK_ORIENTATION_HORIZONTAL, NULL, &wide_nat);
 
-    measure_responses_do (self, for_size >= 0 && for_size < wide_nat,
+    use_compact_layout = for_size >= 0 && wide_nat > for_size &&
+                         !gtk_widget_has_css_class (GTK_WIDGET (self), "short");
+    measure_responses_do (self, use_compact_layout,
                           orientation, minimum, natural);
   }
 
@@ -637,7 +739,7 @@ allocate_responses (GtkWidget *widget,
 
   measure_responses_do (self, FALSE, GTK_ORIENTATION_HORIZONTAL, NULL, &wide_nat);
 
-  compact = wide_nat > width;
+  compact = wide_nat > width && !gtk_widget_has_css_class (GTK_WIDGET (self), "short");
 
   if (compact)
     gtk_widget_add_css_class (widget, "compact");
@@ -652,16 +754,6 @@ allocate_responses (GtkWidget *widget,
       ResponseInfo *response = l->data;
       int child_height;
 
-      if (response->separator) {
-        gtk_widget_measure (response->separator, GTK_ORIENTATION_VERTICAL, -1,
-                            &child_height, NULL, NULL, NULL);
-
-        pos -= child_height;
-
-        gtk_widget_allocate (response->separator, width, child_height, -1,
-                             gsk_transform_translate (NULL, &GRAPHENE_POINT_INIT (0, pos)));
-      }
-
       gtk_widget_measure (response->button, GTK_ORIENTATION_VERTICAL, -1,
                           &child_height, NULL, NULL, NULL);
 
@@ -669,48 +761,19 @@ allocate_responses (GtkWidget *widget,
 
       gtk_widget_allocate (response->button, width, child_height, -1,
                            gsk_transform_translate (NULL, &GRAPHENE_POINT_INIT (0, pos)));
+
+      pos -= BUTTON_SPACING;
     }
   } else {
     gboolean is_rtl = gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL;
     int pos = is_rtl ? width : 0;
     int n_buttons = g_list_length (priv->responses);
-    int total_width = width;
-    int button_width;
+    int total_width = width - BUTTON_SPACING * MAX (0, (n_buttons - 1));
+    int button_width = (int) ceil ((double) total_width / n_buttons);
     GList *l;
 
     for (l = priv->responses; l; l = l->next) {
       ResponseInfo *response = l->data;
-      int separator_width;
-
-      if (!response->separator)
-        continue;
-
-      gtk_widget_measure (response->separator, GTK_ORIENTATION_HORIZONTAL, -1,
-                          &separator_width, NULL, NULL, NULL);
-
-      total_width -= separator_width;
-    }
-
-    button_width = (int) ceil ((double) total_width / n_buttons);
-
-    for (l = priv->responses; l; l = l->next) {
-      ResponseInfo *response = l->data;
-
-      if (response->separator) {
-        int separator_width;
-
-        gtk_widget_measure (response->separator, GTK_ORIENTATION_HORIZONTAL, -1,
-                            &separator_width, NULL, NULL, NULL);
-
-        if (is_rtl)
-          pos -= separator_width;
-
-        gtk_widget_allocate (response->separator, separator_width, height, -1,
-                             gsk_transform_translate (NULL, &GRAPHENE_POINT_INIT (pos, 0)));
-
-        if (!is_rtl)
-          pos += separator_width;
-      }
 
       button_width = MIN (button_width, total_width);
 
@@ -723,7 +786,9 @@ allocate_responses (GtkWidget *widget,
                            gsk_transform_translate (NULL, &GRAPHENE_POINT_INIT (pos, 0)));
 
       if (!is_rtl)
-        pos += button_width;
+        pos += button_width + BUTTON_SPACING;
+      else
+        pos -= BUTTON_SPACING;
     }
   }
 }
@@ -749,26 +814,56 @@ adw_message_dialog_measure (GtkWidget      *widget,
                                                                NULL, NULL);
 
   if (orientation == GTK_ORIENTATION_HORIZONTAL) {
-    int wide_nat, narrow_nat;
+    gboolean is_short = gtk_widget_has_css_class (GTK_WIDGET (self), "short");
+    int wide_nat, narrow_nat, heading_nat;
+    int max_width = adw_length_unit_to_px (ADW_LENGTH_UNIT_SP,
+                                           DIALOG_MAX_WIDTH,
+                                           gtk_widget_get_settings (widget));
+    int pref_width = adw_length_unit_to_px (ADW_LENGTH_UNIT_SP,
+                                            DIALOG_PREFERRED_WIDTH,
+                                            gtk_widget_get_settings (widget));
 
     min_size = MAX (min_size, DIALOG_MIN_WIDTH);
 
     if (priv->parent_window) {
-      max_size = priv->parent_width - DIALOG_MARGIN * 2;
-      max_size = MIN (max_size, DIALOG_MAX_WIDTH);
+      max_size = priv->parent_width - DIALOG_MARGIN_HORZ * 2;
+      max_size = MIN (max_size, max_width);
     } else {
-      max_size = DIALOG_MAX_WIDTH;
+      max_size = max_width;
+    }
+
+    if (gtk_widget_get_visible (priv->heading_bin)) {
+      gtk_widget_measure (priv->heading_bin, orientation, -1,
+                          NULL, &heading_nat, NULL, NULL);
+    } else {
+      heading_nat = 0;
     }
 
     measure_responses_do (self, FALSE, GTK_ORIENTATION_HORIZONTAL, NULL, &wide_nat);
     measure_responses_do (self, TRUE, GTK_ORIENTATION_HORIZONTAL, NULL, &narrow_nat);
 
-    narrow_nat = MAX (narrow_nat, DIALOG_MIN_WIDTH);
+    if (is_short) {
+      wide_nat += RESPONSE_HORZ_PADDING_SHORT;
+      narrow_nat += RESPONSE_HORZ_PADDING_SHORT;
+    } else {
+      wide_nat += RESPONSE_HORZ_PADDING;
+      narrow_nat += RESPONSE_HORZ_PADDING;
+    }
 
-    if (max_size < wide_nat)
+    narrow_nat = MAX (narrow_nat, pref_width);
+
+    if (is_short) {
+      min_size = MAX (min_size, max_size);
+      max_size = adw_length_unit_to_px (ADW_LENGTH_UNIT_SP,
+                                        DIALOG_MAX_WIDE_WIDTH,
+                                        gtk_widget_get_settings (widget));
+
+      max_size = MIN (max_size, wide_nat);
+    } else if (wide_nat > min_size) {
       max_size = MIN (max_size, narrow_nat);
+    }
   } else {
-    max_size = priv->parent_height - DIALOG_MARGIN * 2;
+    max_size = priv->parent_height - DIALOG_MARGIN_VERT * 2;
   }
 
   max_size = MAX (min_size, max_size);
@@ -903,11 +998,12 @@ adw_message_dialog_class_init (AdwMessageDialogClass *klass)
   window_class->close_request = adw_message_dialog_close_request;
 
   /**
-   * AdwMessageDialog:heading: (attributes org.gtk.Property.get=adw_message_dialog_get_heading org.gtk.Property.set=adw_message_dialog_set_heading)
+   * AdwMessageDialog:heading:
    *
    * The heading of the dialog.
    *
    * Since: 1.2
+   * Deprecated: 1.6: Use [class@AlertDialog].
    */
   props[PROP_HEADING] =
     g_param_spec_string ("heading", NULL, NULL,
@@ -915,13 +1011,14 @@ adw_message_dialog_class_init (AdwMessageDialogClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwMessageDialog:heading-use-markup: (attributes org.gtk.Property.get=adw_message_dialog_get_heading_use_markup org.gtk.Property.set=adw_message_dialog_set_heading_use_markup)
+   * AdwMessageDialog:heading-use-markup:
    *
    * Whether the heading includes Pango markup.
    *
    * See [func@Pango.parse_markup].
    *
    * Since: 1.2
+   * Deprecated: 1.6: Use [class@AlertDialog].
    */
   props[PROP_HEADING_USE_MARKUP] =
     g_param_spec_boolean ("heading-use-markup", NULL, NULL,
@@ -929,11 +1026,12 @@ adw_message_dialog_class_init (AdwMessageDialogClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwMessageDialog:body: (attributes org.gtk.Property.get=adw_message_dialog_get_body org.gtk.Property.set=adw_message_dialog_set_body)
+   * AdwMessageDialog:body:
    *
    * The body text of the dialog.
    *
    * Since: 1.2
+   * Deprecated: 1.6: Use [class@AlertDialog].
    */
   props[PROP_BODY] =
     g_param_spec_string ("body", NULL, NULL,
@@ -941,13 +1039,14 @@ adw_message_dialog_class_init (AdwMessageDialogClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwMessageDialog:body-use-markup: (attributes org.gtk.Property.get=adw_message_dialog_get_body_use_markup org.gtk.Property.set=adw_message_dialog_set_body_use_markup)
+   * AdwMessageDialog:body-use-markup:
    *
    * Whether the body text includes Pango markup.
    *
    * See [func@Pango.parse_markup].
    *
    * Since: 1.2
+   * Deprecated: 1.6: Use [class@AlertDialog].
    */
   props[PROP_BODY_USE_MARKUP] =
     g_param_spec_boolean ("body-use-markup", NULL, NULL,
@@ -955,13 +1054,14 @@ adw_message_dialog_class_init (AdwMessageDialogClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwMessageDialog:extra-child: (attributes org.gtk.Property.get=adw_message_dialog_get_extra_child org.gtk.Property.set=adw_message_dialog_set_extra_child)
+   * AdwMessageDialog:extra-child:
    *
    * The child widget.
    *
    * Displayed below the heading and body.
    *
    * Since: 1.2
+   * Deprecated: 1.6: Use [class@AlertDialog].
    */
   props[PROP_EXTRA_CHILD] =
     g_param_spec_object ("extra-child", NULL, NULL,
@@ -969,7 +1069,7 @@ adw_message_dialog_class_init (AdwMessageDialogClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwMessageDialog:default-response: (attributes org.gtk.Property.get=adw_message_dialog_get_default_response org.gtk.Property.set=adw_message_dialog_set_default_response)
+   * AdwMessageDialog:default-response:
    *
    * The response ID of the default response.
    *
@@ -979,6 +1079,7 @@ adw_message_dialog_class_init (AdwMessageDialogClass *klass)
    * will do nothing.
    *
    * Since: 1.2
+   * Deprecated: 1.6: Use [class@AlertDialog].
    */
   props[PROP_DEFAULT_RESPONSE] =
     g_param_spec_string ("default-response", NULL, NULL,
@@ -986,7 +1087,7 @@ adw_message_dialog_class_init (AdwMessageDialogClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwMessageDialog:close-response: (attributes org.gtk.Property.get=adw_message_dialog_get_close_response org.gtk.Property.set=adw_message_dialog_set_close_response)
+   * AdwMessageDialog:close-response:
    *
    * The ID of the close response.
    *
@@ -998,6 +1099,7 @@ adw_message_dialog_class_init (AdwMessageDialogClass *klass)
    * The default close response is `close`.
    *
    * Since: 1.2
+   * Deprecated: 1.6: Use [class@AlertDialog].
    */
   props[PROP_CLOSE_RESPONSE] =
     g_param_spec_string ("close-response", NULL, NULL,
@@ -1021,6 +1123,7 @@ adw_message_dialog_class_init (AdwMessageDialogClass *klass)
    * [property@MessageDialog:close-response].
    *
    * Since: 1.2
+   * Deprecated: 1.6: Use [class@AlertDialog].
    */
   signals[SIGNAL_RESPONSE] =
     g_signal_new ("response",
@@ -1039,8 +1142,11 @@ adw_message_dialog_class_init (AdwMessageDialogClass *klass)
   gtk_widget_class_set_template_from_resource (widget_class,
                                                "/org/gnome/Adwaita/ui/adw-message-dialog.ui");
 
+  gtk_widget_class_bind_template_child_private (widget_class, AdwMessageDialog, heading_bin);
   gtk_widget_class_bind_template_child_private (widget_class, AdwMessageDialog, heading_label);
+  gtk_widget_class_bind_template_child_private (widget_class, AdwMessageDialog, heading_label_small);
   gtk_widget_class_bind_template_child_private (widget_class, AdwMessageDialog, body_label);
+  gtk_widget_class_bind_template_child_private (widget_class, AdwMessageDialog, child_bin);
   gtk_widget_class_bind_template_child_private (widget_class, AdwMessageDialog, message_area);
   gtk_widget_class_bind_template_child_private (widget_class, AdwMessageDialog, response_area);
 
@@ -1070,11 +1176,21 @@ adw_message_dialog_init (AdwMessageDialog *self)
 
   gtk_widget_init_template (GTK_WIDGET (self));
 
+  gtk_widget_set_layout_manager (priv->heading_bin,
+                                 gtk_custom_layout_new (get_heading_request_mode,
+                                                        measure_heading,
+                                                        allocate_heading));
+
   gtk_widget_set_layout_manager (priv->response_area,
                                  gtk_custom_layout_new (get_responses_request_mode,
                                                         measure_responses,
                                                         allocate_responses));
 
+  adw_gizmo_set_focus_func (ADW_GIZMO (priv->heading_bin),
+                            (AdwGizmoFocusFunc) adw_widget_focus_child);
+  adw_gizmo_set_grab_focus_func (ADW_GIZMO (priv->heading_bin),
+                            (AdwGizmoGrabFocusFunc) adw_widget_grab_focus_child);
+
   adw_gizmo_set_focus_func (ADW_GIZMO (priv->response_area),
                             (AdwGizmoFocusFunc) adw_widget_focus_child);
   adw_gizmo_set_grab_focus_func (ADW_GIZMO (priv->response_area),
@@ -1338,6 +1454,7 @@ adw_message_dialog_buildable_init (GtkBuildableIface *iface)
  * Returns: the newly created `AdwMessageDialog`
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 GtkWidget *
 adw_message_dialog_new (GtkWindow  *parent,
@@ -1362,7 +1479,7 @@ adw_message_dialog_new (GtkWindow  *parent,
 }
 
 /**
- * adw_message_dialog_get_heading: (attributes org.gtk.Method.get_property=heading)
+ * adw_message_dialog_get_heading:
  * @self: a message dialog
  *
  * Gets the heading of @self.
@@ -1370,6 +1487,7 @@ adw_message_dialog_new (GtkWindow  *parent,
  * Returns: (nullable): the heading of @self.
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 const char *
 adw_message_dialog_get_heading (AdwMessageDialog *self)
@@ -1384,13 +1502,14 @@ adw_message_dialog_get_heading (AdwMessageDialog *self)
 }
 
 /**
- * adw_message_dialog_set_heading: (attributes org.gtk.Method.set_property=heading)
+ * adw_message_dialog_set_heading:
  * @self: a message dialog
  * @heading: (nullable): the heading of @self
  *
  * Sets the heading of @self.
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_set_heading (AdwMessageDialog *self,
@@ -1403,14 +1522,13 @@ adw_message_dialog_set_heading (AdwMessageDialog *self,
 
   priv = adw_message_dialog_get_instance_private (self);
 
-  if (heading == priv->heading)
+  if (!g_set_str (&priv->heading, heading))
     return;
 
-  g_free (priv->heading);
-  priv->heading = g_strdup (heading);
-
   gtk_label_set_label (GTK_LABEL (priv->heading_label), heading);
-  gtk_widget_set_visible (priv->heading_label, heading && *heading);
+  gtk_label_set_label (GTK_LABEL (priv->heading_label_small), heading);
+
+  gtk_widget_set_visible (priv->heading_bin, heading && *heading);
 
   if (heading && *heading)
     gtk_widget_add_css_class (GTK_WIDGET (priv->message_area), "has-heading");
@@ -1423,7 +1541,7 @@ adw_message_dialog_set_heading (AdwMessageDialog *self,
 }
 
 /**
- * adw_message_dialog_get_heading_use_markup: (attributes org.gtk.Method.get_property=heading-use-markup)
+ * adw_message_dialog_get_heading_use_markup:
  * @self: a message dialog
  *
  * Gets whether the heading of @self includes Pango markup.
@@ -1431,6 +1549,7 @@ adw_message_dialog_set_heading (AdwMessageDialog *self,
  * Returns: whether @self uses markup for heading
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 gboolean
 adw_message_dialog_get_heading_use_markup (AdwMessageDialog *self)
@@ -1445,7 +1564,7 @@ adw_message_dialog_get_heading_use_markup (AdwMessageDialog *self)
 }
 
 /**
- * adw_message_dialog_set_heading_use_markup: (attributes org.gtk.Method.set_property=heading-use-markup)
+ * adw_message_dialog_set_heading_use_markup:
  * @self: a message dialog
  * @use_markup: whether to use markup for heading
  *
@@ -1454,6 +1573,7 @@ adw_message_dialog_get_heading_use_markup (AdwMessageDialog *self)
  * See [func@Pango.parse_markup].
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_set_heading_use_markup (AdwMessageDialog *self,
@@ -1473,6 +1593,7 @@ adw_message_dialog_set_heading_use_markup (AdwMessageDialog *self,
   priv->heading_use_markup = use_markup;
 
   gtk_label_set_use_markup (GTK_LABEL (priv->heading_label), use_markup);
+  gtk_label_set_use_markup (GTK_LABEL (priv->heading_label_small), use_markup);
 
   update_window_title (self);
 
@@ -1491,6 +1612,7 @@ adw_message_dialog_set_heading_use_markup (AdwMessageDialog *self,
  * See [property@MessageDialog:heading].
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_format_heading (AdwMessageDialog *self,
@@ -1540,6 +1662,7 @@ adw_message_dialog_format_heading (AdwMessageDialog *self,
  * See [property@MessageDialog:heading].
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_format_heading_markup (AdwMessageDialog *self,
@@ -1573,7 +1696,7 @@ adw_message_dialog_format_heading_markup (AdwMessageDialog *self,
 }
 
 /**
- * adw_message_dialog_get_body: (attributes org.gtk.Method.get_property=body)
+ * adw_message_dialog_get_body:
  * @self: a message dialog
  *
  * Gets the body text of @self.
@@ -1581,6 +1704,7 @@ adw_message_dialog_format_heading_markup (AdwMessageDialog *self,
  * Returns: the body of @self.
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 const char *
 adw_message_dialog_get_body (AdwMessageDialog *self)
@@ -1595,13 +1719,14 @@ adw_message_dialog_get_body (AdwMessageDialog *self)
 }
 
 /**
- * adw_message_dialog_set_body: (attributes org.gtk.Method.set_property=body)
+ * adw_message_dialog_set_body:
  * @self: a message dialog
  * @body: the body of @self
  *
  * Sets the body text of @self.
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_set_body (AdwMessageDialog *self,
@@ -1614,12 +1739,9 @@ adw_message_dialog_set_body (AdwMessageDialog *self,
 
   priv = adw_message_dialog_get_instance_private (self);
 
-  if (body == priv->body)
+  if (!g_set_str (&priv->body, body))
     return;
 
-  g_free (priv->body);
-  priv->body = g_strdup (body);
-
   gtk_label_set_label (GTK_LABEL (priv->body_label), body);
   gtk_widget_set_visible (priv->body_label, body && *body);
 
@@ -1632,7 +1754,7 @@ adw_message_dialog_set_body (AdwMessageDialog *self,
 }
 
 /**
- * adw_message_dialog_get_body_use_markup: (attributes org.gtk.Method.get_property=body-use-markup)
+ * adw_message_dialog_get_body_use_markup:
  * @self: a message dialog
  *
  * Gets whether the body text of @self includes Pango markup.
@@ -1640,6 +1762,7 @@ adw_message_dialog_set_body (AdwMessageDialog *self,
  * Returns: whether @self uses markup for body text
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 gboolean
 adw_message_dialog_get_body_use_markup (AdwMessageDialog *self)
@@ -1654,7 +1777,7 @@ adw_message_dialog_get_body_use_markup (AdwMessageDialog *self)
 }
 
 /**
- * adw_message_dialog_set_body_use_markup: (attributes org.gtk.Method.set_property=body-use-markup)
+ * adw_message_dialog_set_body_use_markup:
  * @self: a message dialog
  * @use_markup: whether to use markup for body text
  *
@@ -1663,6 +1786,7 @@ adw_message_dialog_get_body_use_markup (AdwMessageDialog *self)
  * See [func@Pango.parse_markup].
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_set_body_use_markup (AdwMessageDialog *self,
@@ -1697,6 +1821,7 @@ adw_message_dialog_set_body_use_markup (AdwMessageDialog *self,
  * See [property@MessageDialog:body].
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_format_body (AdwMessageDialog *self,
@@ -1746,6 +1871,7 @@ adw_message_dialog_format_body (AdwMessageDialog *self,
  * See [property@MessageDialog:body].
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_format_body_markup (AdwMessageDialog *self,
@@ -1779,7 +1905,7 @@ adw_message_dialog_format_body_markup (AdwMessageDialog *self,
 }
 
 /**
- * adw_message_dialog_get_extra_child: (attributes org.gtk.Method.get_property=extra-child)
+ * adw_message_dialog_get_extra_child:
  * @self: a message dialog
  *
  * Gets the child widget of @self.
@@ -1787,6 +1913,7 @@ adw_message_dialog_format_body_markup (AdwMessageDialog *self,
  * Returns: (nullable) (transfer none): the child widget of @self.
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 GtkWidget *
 adw_message_dialog_get_extra_child (AdwMessageDialog *self)
@@ -1801,7 +1928,7 @@ adw_message_dialog_get_extra_child (AdwMessageDialog *self)
 }
 
 /**
- * adw_message_dialog_set_extra_child: (attributes org.gtk.Method.set_property=extra-child)
+ * adw_message_dialog_set_extra_child:
  * @self: a message dialog
  * @child: (nullable): the child widget
  *
@@ -1810,6 +1937,7 @@ adw_message_dialog_get_extra_child (AdwMessageDialog *self)
  * The child widget is displayed below the heading and body.
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_set_extra_child (AdwMessageDialog *self,
@@ -1828,13 +1956,9 @@ adw_message_dialog_set_extra_child (AdwMessageDialog *self,
   if (child == priv->child)
     return;
 
-  if (priv->child)
-    gtk_box_remove (priv->message_area, priv->child);
-
   priv->child = child;
-
-  if (priv->child)
-    gtk_box_append (priv->message_area, priv->child);
+  adw_bin_set_child (ADW_BIN (priv->child_bin), child);
+  gtk_widget_set_visible (priv->child_bin, child != NULL);
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_EXTRA_CHILD]);
 }
@@ -1863,6 +1987,7 @@ adw_message_dialog_set_extra_child (AdwMessageDialog *self,
  * responses further.
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_add_response (AdwMessageDialog *self,
@@ -1892,11 +2017,6 @@ adw_message_dialog_add_response (AdwMessageDialog *self,
   info->appearance = ADW_RESPONSE_DEFAULT;
   info->enabled = TRUE;
 
-  if (priv->responses) {
-    info->separator = gtk_separator_new (GTK_ORIENTATION_VERTICAL);
-    gtk_widget_set_parent (info->separator, priv->response_area);
-  }
-
   info->button = create_response_button (self, info);
   gtk_widget_set_parent (info->button, priv->response_area);
 
@@ -1930,6 +2050,7 @@ adw_message_dialog_add_response (AdwMessageDialog *self,
  * ```
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_add_responses (AdwMessageDialog *self,
@@ -1941,11 +2062,11 @@ adw_message_dialog_add_responses (AdwMessageDialog *self,
 
   g_return_if_fail (ADW_IS_MESSAGE_DIALOG (self));
 
-  va_start (args, first_id);
-
   if (!first_id)
     return;
 
+  va_start (args, first_id);
+
   id = first_id;
   label = va_arg (args, const char *);
 
@@ -1962,6 +2083,47 @@ adw_message_dialog_add_responses (AdwMessageDialog *self,
   va_end (args);
 }
 
+/**
+ * adw_message_dialog_remove_response:
+ * @self: a message dialog
+ * @id: the response ID
+ *
+ * Removes a response from @self.
+ *
+ * Since: 1.5
+ * Deprecated: 1.6: Use [class@AlertDialog].
+ */
+void
+adw_message_dialog_remove_response (AdwMessageDialog *self,
+                                    const char       *id)
+{
+  AdwMessageDialogPrivate *priv;
+  ResponseInfo *info;
+
+  g_return_if_fail (ADW_IS_MESSAGE_DIALOG (self));
+  g_return_if_fail (id != NULL);
+
+  priv = adw_message_dialog_get_instance_private (self);
+  info = find_response (self, id);
+
+  if (!info) {
+    g_critical ("Trying to remove a response with id '%s' from an "
+                "AdwMessageDialog, but such a response does not exist",
+                id);
+    return;
+  }
+
+  if (priv->default_response == info->id)
+    gtk_window_set_default_widget (GTK_WINDOW (self), NULL);
+
+  gtk_widget_unparent (info->button);
+
+  priv->responses = g_list_remove (priv->responses, info);
+  g_hash_table_remove (priv->id_to_response, id);
+
+  response_info_free (info);
+}
+
 /**
  * adw_message_dialog_get_response_label:
  * @self: a message dialog
@@ -1974,6 +2136,7 @@ adw_message_dialog_add_responses (AdwMessageDialog *self,
  * Returns: the label of @response
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 const char *
 adw_message_dialog_get_response_label (AdwMessageDialog *self,
@@ -2002,6 +2165,7 @@ adw_message_dialog_get_response_label (AdwMessageDialog *self,
  * indicates a mnemonic.
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_set_response_label (AdwMessageDialog *self,
@@ -2017,8 +2181,7 @@ adw_message_dialog_set_response_label (AdwMessageDialog *self,
 
   info = find_response (self, response);
 
-  g_free (info->label);
-  info->label = g_strdup (label);
+  g_set_str (&info->label, label);
 
   gtk_button_set_label (GTK_BUTTON (info->button), label);
 }
@@ -2035,6 +2198,7 @@ adw_message_dialog_set_response_label (AdwMessageDialog *self,
  * Returns: the appearance of @response
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 AdwResponseAppearance
 adw_message_dialog_get_response_appearance (AdwMessageDialog *self,
@@ -2076,6 +2240,7 @@ adw_message_dialog_get_response_appearance (AdwMessageDialog *self,
  * Negative responses like Cancel or Close should use the default appearance.
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_set_response_appearance (AdwMessageDialog      *self,
@@ -2098,14 +2263,14 @@ adw_message_dialog_set_response_appearance (AdwMessageDialog      *self,
   info->appearance = appearance;
 
   if (info->appearance == ADW_RESPONSE_SUGGESTED)
-    gtk_widget_add_css_class (info->button, "suggested");
+    gtk_widget_add_css_class (info->button, "suggested-action");
   else
-    gtk_widget_remove_css_class (info->button, "suggested");
+    gtk_widget_remove_css_class (info->button, "suggested-action");
 
   if (info->appearance == ADW_RESPONSE_DESTRUCTIVE)
-    gtk_widget_add_css_class (info->button, "destructive");
+    gtk_widget_add_css_class (info->button, "destructive-action");
   else
-    gtk_widget_remove_css_class (info->button, "destructive");
+    gtk_widget_remove_css_class (info->button, "destructive-action");
 }
 
 /**
@@ -2120,6 +2285,7 @@ adw_message_dialog_set_response_appearance (AdwMessageDialog      *self,
  * Returns: whether @response is enabled
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 gboolean
 adw_message_dialog_get_response_enabled (AdwMessageDialog *self,
@@ -2154,6 +2320,7 @@ adw_message_dialog_get_response_enabled (AdwMessageDialog *self,
  * Responses are enabled by default.
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_set_response_enabled (AdwMessageDialog *self,
@@ -2179,7 +2346,7 @@ adw_message_dialog_set_response_enabled (AdwMessageDialog *self,
 }
 
 /**
- * adw_message_dialog_get_default_response: (attributes org.gtk.Method.get_property=default-response)
+ * adw_message_dialog_get_default_response:
  * @self: a message dialog
  *
  * Gets the ID of the default response of @self.
@@ -2187,6 +2354,7 @@ adw_message_dialog_set_response_enabled (AdwMessageDialog *self,
  * Returns: (nullable): the default response ID
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 const char *
 adw_message_dialog_get_default_response (AdwMessageDialog *self)
@@ -2204,7 +2372,7 @@ adw_message_dialog_get_default_response (AdwMessageDialog *self)
 }
 
 /**
- * adw_message_dialog_set_default_response: (attributes org.gtk.Method.set_property=default-response)
+ * adw_message_dialog_set_default_response:
  * @self: a message dialog
  * @response: (nullable): the default response ID
  *
@@ -2216,6 +2384,7 @@ adw_message_dialog_get_default_response (AdwMessageDialog *self)
  * will do nothing.
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_set_default_response (AdwMessageDialog *self,
@@ -2244,7 +2413,7 @@ adw_message_dialog_set_default_response (AdwMessageDialog *self,
 }
 
 /**
- * adw_message_dialog_get_close_response: (attributes org.gtk.Method.get_property=close-response)
+ * adw_message_dialog_get_close_response:
  * @self: a message dialog
  *
  * Gets the ID of the close response of @self.
@@ -2252,6 +2421,7 @@ adw_message_dialog_set_default_response (AdwMessageDialog *self,
  * Returns: the close response ID
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 const char *
 adw_message_dialog_get_close_response (AdwMessageDialog *self)
@@ -2266,7 +2436,7 @@ adw_message_dialog_get_close_response (AdwMessageDialog *self)
 }
 
 /**
- * adw_message_dialog_set_close_response: (attributes org.gtk.Method.set_property=close-response)
+ * adw_message_dialog_set_close_response:
  * @self: a message dialog
  * @response: the close response ID
  *
@@ -2280,6 +2450,7 @@ adw_message_dialog_get_close_response (AdwMessageDialog *self)
  * The default close response is `close`.
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_set_close_response (AdwMessageDialog *self,
@@ -2312,6 +2483,7 @@ adw_message_dialog_set_close_response (AdwMessageDialog *self,
  * Used to indicate that the user has responded to the dialog in some way.
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_response (AdwMessageDialog *self,
@@ -2334,6 +2506,7 @@ adw_message_dialog_response (AdwMessageDialog *self,
  * Returns: whether @self has a response with the ID @response.
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 gboolean
 adw_message_dialog_has_response (AdwMessageDialog *self,
@@ -2376,17 +2549,15 @@ choose_cancelled_cb (GCancellable *cancellable,
 
 /**
  * adw_message_dialog_choose:
- * @self: (transfer full): a message dialog
+ * @self: a message dialog
  * @cancellable: (nullable): a `GCancellable` to cancel the operation
  * @callback: (scope async): a callback to call when the operation is complete
  * @user_data: (closure callback): data to pass to @callback
  *
  * This function shows @self to the user.
  *
- * The @callback will be called when the alert is dismissed. It should call
- * [method@MessageDialog.choose_finish] to obtain the result.
- *
  * Since: 1.3
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_choose (AdwMessageDialog    *self,
@@ -2420,6 +2591,7 @@ adw_message_dialog_choose (AdwMessageDialog    *self,
  *   [property@MessageDialog:close-response] if the call was cancelled.
  *
  * Since: 1.3
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 const char *
 adw_message_dialog_choose_finish (AdwMessageDialog *self,
@@ -2434,3 +2606,4 @@ adw_message_dialog_choose_finish (AdwMessageDialog *self,
 
   return g_quark_to_string (id);
 }
+
