@@ -10,6 +10,7 @@
 
 #include "adw-style-manager-private.h"
 
+#include "adw-accent-color-private.h"
 #include "adw-main-private.h"
 #include "adw-settings-private.h"
 #include <gtk/gtk.h>
@@ -37,11 +38,13 @@
  * A class for managing application-wide styling.
  *
  * `AdwStyleManager` provides a way to query and influence the application
- * styles, such as whether to use dark or high contrast appearance.
+ * styles, such as whether to use dark style, the system accent color or high
+ * contrast appearance.
  *
  * It allows to set the color scheme via the
  * [property@StyleManager:color-scheme] property, and to query the current
- * appearance, as well as whether a system-wide color scheme preference exists.
+ * appearance, as well as whether a system-wide color scheme and accent color
+ * preferences exists.
  */
 
 struct _AdwStyleManager
@@ -52,6 +55,7 @@ struct _AdwStyleManager
   AdwSettings *settings;
   GtkCssProvider *provider;
   GtkCssProvider *colors_provider;
+  GtkCssProvider *accent_provider;
 
   AdwColorScheme color_scheme;
   gboolean dark;
@@ -70,6 +74,9 @@ enum {
   PROP_SYSTEM_SUPPORTS_COLOR_SCHEMES,
   PROP_DARK,
   PROP_HIGH_CONTRAST,
+  PROP_SYSTEM_SUPPORTS_ACCENT_COLORS,
+  PROP_ACCENT_COLOR,
+  PROP_ACCENT_COLOR_RGBA,
   LAST_PROP,
 };
 
@@ -117,15 +124,32 @@ register_display (GdkDisplayManager *display_manager,
                     NULL);
 }
 
-static gboolean
+static void
 enable_animations_cb (AdwStyleManager *self)
 {
   gtk_style_context_remove_provider_for_display (self->display,
                                                  GTK_STYLE_PROVIDER (self->animations_provider));
 
   self->animation_timeout_id = 0;
+}
 
-  return G_SOURCE_REMOVE;
+static char*
+generate_accent_css (AdwStyleManager *self)
+{
+  AdwAccentColor accent = adw_style_manager_get_accent_color (self);
+  GString *str = g_string_new ("");
+  GdkRGBA rgba;
+  char *rgba_str;
+
+  adw_accent_color_to_rgba (accent, &rgba);
+  rgba_str = gdk_rgba_to_string (&rgba);
+
+  g_string_append_printf (str, "@define-color accent_bg_color %s;\n", rgba_str);
+  g_string_append (str, "@define-color accent_fg_color white;\n");
+
+  g_free (rgba_str);
+
+  return g_string_free (str, FALSE);
 }
 
 static void
@@ -171,10 +195,16 @@ update_stylesheet (AdwStyleManager *self)
                                            "/org/gnome/Adwaita/styles/defaults-light.css");
   }
 
+  if (self->accent_provider) {
+    char *accent_css = generate_accent_css (self);
+    gtk_css_provider_load_from_string (self->accent_provider, accent_css);
+    g_free (accent_css);
+  }
+
   self->animation_timeout_id =
-    g_timeout_add (SWITCH_DURATION,
-                   G_SOURCE_FUNC (enable_animations_cb),
-                   self);
+    g_timeout_add_once (SWITCH_DURATION,
+                        (GSourceOnceFunc) enable_animations_cb,
+                        self);
 }
 
 static gboolean
@@ -221,6 +251,21 @@ notify_system_supports_color_schemes_cb (AdwStyleManager *self)
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_SYSTEM_SUPPORTS_COLOR_SCHEMES]);
 }
 
+static void
+notify_accent_color_cb (AdwStyleManager *self)
+{
+  update_stylesheet (self);
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_ACCENT_COLOR]);
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_ACCENT_COLOR_RGBA]);
+}
+
+static void
+notify_system_supports_accent_colors_cb (AdwStyleManager *self)
+{
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_SYSTEM_SUPPORTS_ACCENT_COLORS]);
+}
+
 static void
 notify_high_contrast_cb (AdwStyleManager *self)
 {
@@ -267,6 +312,11 @@ adw_style_manager_constructed (GObject *object)
       gtk_style_context_add_provider_for_display (self->display,
                                                   GTK_STYLE_PROVIDER (self->colors_provider),
                                                   GTK_STYLE_PROVIDER_PRIORITY_THEME);
+
+      self->accent_provider = gtk_css_provider_new ();
+      gtk_style_context_add_provider_for_display (self->display,
+                                                  GTK_STYLE_PROVIDER (self->accent_provider),
+                                                  GTK_STYLE_PROVIDER_PRIORITY_THEME);
     }
 
     self->animations_provider = gtk_css_provider_new ();
@@ -286,6 +336,16 @@ adw_style_manager_constructed (GObject *object)
                            G_CALLBACK (update_dark),
                            self,
                            G_CONNECT_SWAPPED);
+  g_signal_connect_object (self->settings,
+                           "notify::system-supports-accent-colors",
+                           G_CALLBACK (notify_system_supports_accent_colors_cb),
+                           self,
+                           G_CONNECT_SWAPPED);
+  g_signal_connect_object (self->settings,
+                           "notify::accent-color",
+                           G_CALLBACK (notify_accent_color_cb),
+                           self,
+                           G_CONNECT_SWAPPED);
   g_signal_connect_object (self->settings,
                            "notify::high-contrast",
                            G_CALLBACK (notify_high_contrast_cb),
@@ -305,6 +365,7 @@ adw_style_manager_dispose (GObject *object)
   g_clear_object (&self->provider);
   g_clear_object (&self->colors_provider);
   g_clear_object (&self->animations_provider);
+  g_clear_object (&self->accent_provider);
 
   G_OBJECT_CLASS (adw_style_manager_parent_class)->dispose (object);
 }
@@ -338,6 +399,18 @@ adw_style_manager_get_property (GObject    *object,
     g_value_set_boolean (value, adw_style_manager_get_high_contrast (self));
     break;
 
+  case PROP_SYSTEM_SUPPORTS_ACCENT_COLORS:
+    g_value_set_boolean (value, adw_style_manager_get_system_supports_accent_colors (self));
+    break;
+
+  case PROP_ACCENT_COLOR:
+    g_value_set_enum (value, adw_style_manager_get_accent_color (self));
+    break;
+
+  case PROP_ACCENT_COLOR_RGBA:
+    g_value_take_boxed (value, adw_style_manager_get_accent_color_rgba (self));
+    break;
+
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
@@ -376,7 +449,7 @@ adw_style_manager_class_init (AdwStyleManagerClass *klass)
   object_class->set_property = adw_style_manager_set_property;
 
   /**
-   * AdwStyleManager:display: (attributes org.gtk.Property.get=adw_style_manager_get_display)
+   * AdwStyleManager:display:
    *
    * The display the style manager is associated with.
    *
@@ -389,7 +462,7 @@ adw_style_manager_class_init (AdwStyleManagerClass *klass)
                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
 
   /**
-   * AdwStyleManager:color-scheme: (attributes org.gtk.Property.get=adw_style_manager_get_color_scheme org.gtk.Property.set=adw_style_manager_set_color_scheme)
+   * AdwStyleManager:color-scheme:
    *
    * The requested application color scheme.
    *
@@ -430,7 +503,7 @@ adw_style_manager_class_init (AdwStyleManagerClass *klass)
                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwStyleManager:system-supports-color-schemes: (attributes org.gtk.Property.get=adw_style_manager_get_system_supports_color_schemes)
+   * AdwStyleManager:system-supports-color-schemes:
    *
    * Whether the system supports color schemes.
    *
@@ -446,7 +519,7 @@ adw_style_manager_class_init (AdwStyleManagerClass *klass)
                           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * AdwStyleManager:dark: (attributes org.gtk.Property.get=adw_style_manager_get_dark)
+   * AdwStyleManager:dark:
    *
    * Whether the application is using dark appearance.
    *
@@ -459,7 +532,7 @@ adw_style_manager_class_init (AdwStyleManagerClass *klass)
                           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * AdwStyleManager:high-contrast: (attributes org.gtk.Property.get=adw_style_manager_get_high_contrast)
+   * AdwStyleManager:high-contrast:
    *
    * Whether the application is using high contrast appearance.
    *
@@ -470,6 +543,56 @@ adw_style_manager_class_init (AdwStyleManagerClass *klass)
                           FALSE,
                           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
+  /**
+   * AdwStyleManager:system-supports-accent-colors:
+   *
+   * Whether the system supports accent colors.
+   *
+   * This property can be used to check if the current environment provides an
+   * accent color preference. For example, applications might want to show a
+   * preference for choosing accent color if it's set to `FALSE`.
+   *
+   * See [property@StyleManager:accent-color].
+   *
+   * Since: 1.6
+   */
+  props[PROP_SYSTEM_SUPPORTS_ACCENT_COLORS] =
+    g_param_spec_boolean ("system-supports-accent-colors", NULL, NULL,
+                          FALSE,
+                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+
+  /**
+   * AdwStyleManager:accent-color:
+   *
+   * The current system accent color.
+   *
+   * See also [property@StyleManager:accent-color-rgba].
+   *
+   * Since: 1.6
+   */
+  props[PROP_ACCENT_COLOR] =
+    g_param_spec_enum ("accent-color", NULL, NULL,
+                       ADW_TYPE_ACCENT_COLOR,
+                       ADW_ACCENT_COLOR_BLUE,
+                       G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+
+  /**
+   * AdwStyleManager:accent-color-rgba:
+   *
+   * The current system accent color as a `GdkRGBA`.
+   *
+   * Equivalent to calling [func@AccentColor.to_rgba] on the value of
+   * [property@StyleManager:accent-color].
+   *
+   * This is a background color. The matching foreground color is white.
+   *
+   * Since: 1.6
+   */
+  props[PROP_ACCENT_COLOR_RGBA] =
+    g_param_spec_boxed ("accent-color-rgba", NULL, NULL,
+                        GDK_TYPE_RGBA,
+                        G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+
   g_object_class_install_properties (object_class, LAST_PROP, props);
 }
 
@@ -556,7 +679,7 @@ adw_style_manager_get_for_display (GdkDisplay *display)
 }
 
 /**
- * adw_style_manager_get_display: (attributes org.gtk.Method.get_property=display)
+ * adw_style_manager_get_display:
  * @self: a style manager
  *
  * Gets the display the style manager is associated with.
@@ -575,7 +698,7 @@ adw_style_manager_get_display (AdwStyleManager *self)
 }
 
 /**
- * adw_style_manager_get_color_scheme: (attributes org.gtk.Method.get_property=color-scheme)
+ * adw_style_manager_get_color_scheme:
  * @self: a style manager
  *
  * Gets the requested application color scheme.
@@ -591,7 +714,7 @@ adw_style_manager_get_color_scheme (AdwStyleManager *self)
 }
 
 /**
- * adw_style_manager_set_color_scheme: (attributes org.gtk.Method.set_property=color-scheme)
+ * adw_style_manager_set_color_scheme:
  * @self: a style manager
  * @color_scheme: the color scheme
  *
@@ -659,7 +782,7 @@ adw_style_manager_set_color_scheme (AdwStyleManager *self,
 }
 
 /**
- * adw_style_manager_get_system_supports_color_schemes: (attributes org.gtk.Method.get_property=system-supports-color-schemes)
+ * adw_style_manager_get_system_supports_color_schemes:
  * @self: a style manager
  *
  * Gets whether the system supports color schemes.
@@ -679,7 +802,7 @@ adw_style_manager_get_system_supports_color_schemes (AdwStyleManager *self)
 }
 
 /**
- * adw_style_manager_get_dark: (attributes org.gtk.Method.get_property=dark)
+ * adw_style_manager_get_dark:
  * @self: a style manager
  *
  * Gets whether the application is using dark appearance.
@@ -698,7 +821,7 @@ adw_style_manager_get_dark (AdwStyleManager *self)
 }
 
 /**
- * adw_style_manager_get_high_contrast: (attributes org.gtk.Method.get_property=high-contrast)
+ * adw_style_manager_get_high_contrast:
  * @self: a style manager
  *
  * Gets whether the application is using high contrast appearance.
@@ -714,3 +837,77 @@ adw_style_manager_get_high_contrast (AdwStyleManager *self)
 
   return adw_settings_get_high_contrast (self->settings);
 }
+
+/**
+ * adw_style_manager_get_system_supports_accent_colors:
+ * @self: a style manager
+ *
+ * Gets whether the system supports accent colors.
+ *
+ * This can be used to check if the current environment provides an accent color
+ * preference. For example, applications might want to show a preference for
+ * choosing accent color if it's set to `FALSE`.
+ *
+ * See [property@StyleManager:accent-color].
+ *
+ * Returns: whether the system supports accent colors
+ *
+ * Since: 1.6
+ */
+gboolean
+adw_style_manager_get_system_supports_accent_colors (AdwStyleManager *self)
+{
+  g_return_val_if_fail (ADW_IS_STYLE_MANAGER (self), FALSE);
+
+  return adw_settings_get_system_supports_accent_colors (self->settings);
+}
+
+/**
+ * adw_style_manager_get_accent_color:
+ * @self: a style manager
+ *
+ * Gets the current system accent color.
+ *
+ * See also [property@StyleManager:accent-color-rgba].
+ *
+ * Returns: the current system accent color
+ *
+ * Since: 1.6
+ */
+AdwAccentColor
+adw_style_manager_get_accent_color (AdwStyleManager *self)
+{
+  g_return_val_if_fail (ADW_IS_STYLE_MANAGER (self), ADW_ACCENT_COLOR_BLUE);
+
+  return adw_settings_get_accent_color (self->settings);
+}
+
+/**
+ * adw_style_manager_get_accent_color_rgba:
+ * @self: a style manager
+ *
+ * Gets the current system accent color as a `GdkRGBA`.
+ *
+ * Equivalent to calling [func@AccentColor.to_rgba] on the value of
+ * [property@StyleManager:accent-color].
+ *
+ * This is a background color. The matching foreground color is white.
+ *
+ * Returns: the current system accent color
+ *
+ * Since: 1.6
+ */
+GdkRGBA *
+adw_style_manager_get_accent_color_rgba (AdwStyleManager *self)
+{
+  AdwAccentColor color;
+  GdkRGBA rgba;
+
+  g_return_val_if_fail (ADW_IS_STYLE_MANAGER (self), NULL);
+
+  color = adw_style_manager_get_accent_color (self);
+
+  adw_accent_color_to_rgba (color, &rgba);
+
+  return gdk_rgba_copy (&rgba);
+}
