@@ -163,6 +163,12 @@ enum {
   SIGNAL_LAST_SIGNAL,
 };
 
+typedef enum {
+  ANIMATION_NONE,
+  ANIMATION_IN,
+  ANIMATION_OUT,
+} AnimationDirection;
+
 static guint signals[SIGNAL_LAST_SIGNAL];
 
 #define ADW_TYPE_TAB_OVERVIEW_SCROLLABLE (adw_tab_overview_scrollable_get_type ())
@@ -534,11 +540,15 @@ adw_tab_overview_scrollable_size_allocate (GtkWidget *widget,
   adw_tab_grid_set_visible_range (ADW_TAB_GRID (self->pinned_grid),
                                   CLAMP (value - self->pinned_grid_pos, 0, pinned_height),
                                   CLAMP (value - self->pinned_grid_pos + height - new_button_height, 0, pinned_height),
-                                  height - new_button_height);
+                                  height - new_button_height,
+                                  0,
+                                  CLAMP (self->pinned_grid_pos + pinned_height - height + new_button_height - value, 0, new_button_height));
   adw_tab_grid_set_visible_range (ADW_TAB_GRID (self->grid),
                                   CLAMP (value - self->grid_pos, 0, grid_height),
                                   CLAMP (value - self->grid_pos + height - new_button_height, 0, grid_height),
-                                  height - new_button_height);
+                                  height - new_button_height,
+                                  0,
+                                  CLAMP (self->grid_pos + grid_height - height + new_button_height - value, 0, new_button_height));
 
   if (self->scroll_animation_done) {
     g_clear_pointer (&self->scroll_animation_grid, adw_tab_grid_reset_scrolled_tab);
@@ -915,7 +925,7 @@ notify_n_pages_cb (AdwTabOverview *self)
   n_pages = adw_tab_view_get_n_pages (self->view);
 
   /* Translators: Tab overview title, %u is the number of open tabs */
-  title_str = g_strdup_printf (ngettext ("%u Tab", "%u Tabs", n_pages), n_pages);
+  title_str = g_strdup_printf (dngettext (GETTEXT_PACKAGE, "%u Tab", "%u Tabs", n_pages), n_pages);
 
   adw_window_title_set_title (ADW_WINDOW_TITLE (self->title), title_str);
 
@@ -974,15 +984,17 @@ open_animation_value_cb (double          value,
 }
 
 static void
-set_overview_visible (AdwTabOverview *self,
-                      gboolean        visible,
-                      gboolean        animating)
+set_overview_visible (AdwTabOverview     *self,
+                      gboolean            visible,
+                      AnimationDirection  direction)
 {
+  gboolean animating = direction != ANIMATION_NONE;
+
   gtk_widget_set_child_visible (self->overview, visible || animating);
   gtk_widget_set_can_target (self->overview, visible);
   gtk_widget_set_can_focus (self->overview, visible);
   gtk_widget_set_can_target (self->child_bin, !visible && !animating);
-  gtk_widget_set_can_focus (self->child_bin, !visible && !animating);
+  gtk_widget_set_can_focus (self->child_bin, !visible && direction != ANIMATION_IN);
 
   if (visible || animating)
     gtk_widget_add_css_class (self->child_bin, "background");
@@ -1000,7 +1012,7 @@ open_animation_done_cb (AdwTabOverview *self)
     self->transition_thumbnail = NULL;
   }
 
-  set_overview_visible (self, self->is_open, FALSE);
+  set_overview_visible (self, self->is_open, ANIMATION_NONE);
 
   if (!self->is_open) {
     adw_tab_view_close_overview (self->view);
@@ -1009,10 +1021,7 @@ open_animation_done_cb (AdwTabOverview *self)
 
     if (self->last_focus) {
       gtk_widget_grab_focus (self->last_focus);
-
-      g_object_remove_weak_pointer (G_OBJECT (self->last_focus),
-                                    (gpointer *) &self->last_focus);
-      self->last_focus = NULL;
+      g_clear_weak_pointer (&self->last_focus);
     }
   }
 
@@ -1352,11 +1361,7 @@ adw_tab_overview_dispose (GObject *object)
 {
   AdwTabOverview *self = ADW_TAB_OVERVIEW (object);
 
-  if (self->last_focus) {
-    g_object_remove_weak_pointer (G_OBJECT (self->last_focus),
-                                  (gpointer *) &self->last_focus);
-    self->last_focus = NULL;
-  }
+  g_clear_weak_pointer (&self->last_focus);
 
   adw_tab_overview_set_view (self, NULL);
 
@@ -1524,7 +1529,7 @@ adw_tab_overview_class_init (AdwTabOverviewClass *klass)
   widget_class->focus = adw_tab_overview_focus;
 
   /**
-   * AdwTabOverview:view: (attributes org.gtk.Property.get=adw_tab_overview_get_view org.gtk.Property.set=adw_tab_overview_set_view)
+   * AdwTabOverview:view:
    *
    * The tab view the overview controls.
    *
@@ -1538,7 +1543,7 @@ adw_tab_overview_class_init (AdwTabOverviewClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwTabOverview:child: (attributes org.gtk.Property.get=adw_tab_overview_get_child org.gtk.Property.set=adw_tab_overview_set_child)
+   * AdwTabOverview:child:
    *
    * The child widget.
    *
@@ -1550,7 +1555,7 @@ adw_tab_overview_class_init (AdwTabOverviewClass *klass)
                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwTabOverview:open: (attributes org.gtk.Property.get=adw_tab_overview_get_open org.gtk.Property.set=adw_tab_overview_set_open)
+   * AdwTabOverview:open:
    *
    * Whether the overview is open.
    *
@@ -1562,7 +1567,7 @@ adw_tab_overview_class_init (AdwTabOverviewClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwTabOverview:inverted: (attributes org.gtk.Property.get=adw_tab_overview_get_inverted org.gtk.Property.set=adw_tab_overview_set_inverted)
+   * AdwTabOverview:inverted:
    *
    * Whether thumbnails use inverted layout.
    *
@@ -1577,7 +1582,7 @@ adw_tab_overview_class_init (AdwTabOverviewClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwTabOverview:enable-search: (attributes org.gtk.Property.get=adw_tab_overview_get_enable_search org.gtk.Property.set=adw_tab_overview_set_enable_search)
+   * AdwTabOverview:enable-search:
    *
    * Whether to enable search in tabs.
    *
@@ -1598,7 +1603,7 @@ adw_tab_overview_class_init (AdwTabOverviewClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwTabOverview:search-active: (attributes org.gtk.Property.get=adw_tab_overview_get_search_active)
+   * AdwTabOverview:search-active:
    *
    * Whether search is currently active.
    *
@@ -1612,7 +1617,7 @@ adw_tab_overview_class_init (AdwTabOverviewClass *klass)
                           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * AdwTabOverview:enable-new-tab: (attributes org.gtk.Property.get=adw_tab_overview_get_enable_new_tab org.gtk.Property.set=adw_tab_overview_set_enable_new_tab)
+   * AdwTabOverview:enable-new-tab:
    *
    * Whether to enable new tab button.
    *
@@ -1626,7 +1631,7 @@ adw_tab_overview_class_init (AdwTabOverviewClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwTabOverview:secondary-menu: (attributes org.gtk.Property.get=adw_tab_overview_get_secondary_menu org.gtk.Property.set=adw_tab_overview_set_secondary_menu)
+   * AdwTabOverview:secondary-menu:
    *
    * The secondary menu model.
    *
@@ -1640,7 +1645,7 @@ adw_tab_overview_class_init (AdwTabOverviewClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwTabOverview:show-start-title-buttons: (attributes org.gtk.Property.get=adw_tab_overview_get_show_start_title_buttons org.gtk.Property.set=adw_tab_overview_set_show_start_title_buttons)
+   * AdwTabOverview:show-start-title-buttons:
    *
    * Whether to show start title buttons in the overview's header bar.
    *
@@ -1654,7 +1659,7 @@ adw_tab_overview_class_init (AdwTabOverviewClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwTabOverview:show-end-title-buttons: (attributes org.gtk.Property.get=adw_tab_overview_get_show_end_title_buttons org.gtk.Property.set=adw_tab_overview_set_show_end_title_buttons)
+   * AdwTabOverview:show-end-title-buttons:
    *
    * Whether to show end title buttons in the overview's header bar.
    *
@@ -1668,7 +1673,7 @@ adw_tab_overview_class_init (AdwTabOverviewClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwTabOverview:extra-drag-preferred-action: (attributes org.gtk.Property.get=adw_tab_overview_get_extra_drag_preferred_action)
+   * AdwTabOverview:extra-drag-preferred-action:
    *
    * The unique action on the `current-drop` of the
    * [signal@TabOverview::extra-drag-drop].
@@ -1685,7 +1690,7 @@ adw_tab_overview_class_init (AdwTabOverviewClass *klass)
                        G_PARAM_READABLE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwTabOverview:extra-drag-preload: (attributes org.gtk.Property.get=adw_tab_overview_get_extra_drag_preload org.gtk.Property.set=adw_tab_overview_set_extra_drag_preload)
+   * AdwTabOverview:extra-drag-preload:
    *
    * Whether the drop data should be preloaded on hover.
    *
@@ -1899,7 +1904,7 @@ adw_tab_overview_new (void)
 }
 
 /**
- * adw_tab_overview_get_view: (attributes org.gtk.Method.get_property=view)
+ * adw_tab_overview_get_view:
  * @self: a tab overview
  *
  * Gets the tab view @self controls.
@@ -1917,7 +1922,7 @@ adw_tab_overview_get_view (AdwTabOverview *self)
 }
 
 /**
- * adw_tab_overview_set_view: (attributes org.gtk.Method.set_property=view)
+ * adw_tab_overview_set_view:
  * @self: a tab overview
  * @view: (nullable): a tab view
  *
@@ -1995,7 +2000,7 @@ adw_tab_overview_set_view (AdwTabOverview *self,
 }
 
 /**
- * adw_tab_overview_get_child: (attributes org.gtk.Method.get_property=child)
+ * adw_tab_overview_get_child:
  * @self: a `AdwTabOveview`
  *
  * Gets the child widget of @self.
@@ -2013,7 +2018,7 @@ adw_tab_overview_get_child (AdwTabOverview *self)
 }
 
 /**
- * adw_tab_overview_set_child: (attributes org.gtk.Method.set_property=child)
+ * adw_tab_overview_set_child:
  * @self: a tab overview
  * @child: (nullable): the child widget
  *
@@ -2040,7 +2045,7 @@ adw_tab_overview_set_child (AdwTabOverview *self,
 }
 
 /**
- * adw_tab_overview_get_open: (attributes org.gtk.Method.get_property=open)
+ * adw_tab_overview_get_open:
  * @self: a tab overview
  *
  * Gets whether @self is open.
@@ -2058,7 +2063,7 @@ adw_tab_overview_get_open (AdwTabOverview *self)
 }
 
 /**
- * adw_tab_overview_set_open: (attributes org.gtk.Method.set_property=open)
+ * adw_tab_overview_set_open:
  * @self: a tab overview
  * @open: whether the overview is open
  *
@@ -2117,24 +2122,16 @@ adw_tab_overview_set_open (AdwTabOverview *self,
     if (gtk_widget_get_root (GTK_WIDGET (self)))
       focus = gtk_root_get_focus (gtk_widget_get_root (GTK_WIDGET (self)));
 
-    if (focus && gtk_widget_is_ancestor (focus, self->child_bin)) {
-      if (self->last_focus)
-        g_object_remove_weak_pointer (G_OBJECT (self->last_focus),
-                                      (gpointer *)& self->last_focus);
-
-      self->last_focus = focus;
-
-      g_object_add_weak_pointer (G_OBJECT (self->last_focus),
-                                 (gpointer *) &self->last_focus);
-    }
+    if (focus && gtk_widget_is_ancestor (focus, self->child_bin))
+      g_set_weak_pointer (&self->last_focus, focus);
 
     adw_tab_view_open_overview (self->view);
 
-    set_overview_visible (self, self->is_open, TRUE);
+    set_overview_visible (self, self->is_open, ANIMATION_IN);
 
     adw_tab_grid_try_focus_selected_tab (grid, FALSE);
   } else {
-    set_overview_visible (self, self->is_open, TRUE);
+    set_overview_visible (self, self->is_open, ANIMATION_OUT);
   }
 
   if (self->transition_picture)
@@ -2156,7 +2153,7 @@ adw_tab_overview_set_open (AdwTabOverview *self,
 }
 
 /**
- * adw_tab_overview_get_inverted: (attributes org.gtk.Method.get_property=inverted)
+ * adw_tab_overview_get_inverted:
  * @self: a tab overview
  *
  * Gets whether thumbnails use inverted layout.
@@ -2174,7 +2171,7 @@ adw_tab_overview_get_inverted (AdwTabOverview *self)
 }
 
 /**
- * adw_tab_overview_set_inverted: (attributes org.gtk.Method.set_property=inverted)
+ * adw_tab_overview_set_inverted:
  * @self: a tab overview
  * @inverted: whether thumbnails use inverted layout
  *
@@ -2203,7 +2200,7 @@ adw_tab_overview_set_inverted (AdwTabOverview *self,
 }
 
 /**
- * adw_tab_overview_get_enable_search: (attributes org.gtk.Method.get_property=enable-search)
+ * adw_tab_overview_get_enable_search:
  * @self: a tab overview
  *
  * Gets whether search in tabs is enabled for @self.
@@ -2221,7 +2218,7 @@ adw_tab_overview_get_enable_search (AdwTabOverview *self)
 }
 
 /**
- * adw_tab_overview_set_enable_search: (attributes org.gtk.Method.set_property=enable-search)
+ * adw_tab_overview_set_enable_search:
  * @self: a tab overview
  * @enable_search: whether to enable search
  *
@@ -2263,7 +2260,7 @@ adw_tab_overview_set_enable_search (AdwTabOverview *self,
 }
 
 /**
- * adw_tab_overview_get_search_active: (attributes org.gtk.Method.get_property=search-active)
+ * adw_tab_overview_get_search_active:
  * @self: a tab overview
  *
  * Gets whether search is currently active for @self.
@@ -2283,7 +2280,7 @@ adw_tab_overview_get_search_active (AdwTabOverview *self)
 }
 
 /**
- * adw_tab_overview_get_enable_new_tab: (attributes org.gtk.Method.get_property=enable-new-tab)
+ * adw_tab_overview_get_enable_new_tab:
  * @self: a tab overview
  *
  * Gets whether to new tab button is enabled for @self.
@@ -2301,7 +2298,7 @@ adw_tab_overview_get_enable_new_tab (AdwTabOverview *self)
 }
 
 /**
- * adw_tab_overview_set_enable_new_tab: (attributes org.gtk.Method.set_property=enable-new-tab)
+ * adw_tab_overview_set_enable_new_tab:
  * @self: a tab overview
  * @enable_new_tab: whether to enable new tab button
  *
@@ -2330,7 +2327,7 @@ adw_tab_overview_set_enable_new_tab (AdwTabOverview *self,
 }
 
 /**
- * adw_tab_overview_get_secondary_menu: (attributes org.gtk.Method.get_property=secondary-menu)
+ * adw_tab_overview_get_secondary_menu:
  * @self: a tab overview
  *
  * Gets the secondary menu model for @self.
@@ -2348,7 +2345,7 @@ adw_tab_overview_get_secondary_menu (AdwTabOverview *self)
 }
 
 /**
- * adw_tab_overview_set_secondary_menu: (attributes org.gtk.Method.set_property=secondary-menu)
+ * adw_tab_overview_set_secondary_menu:
  * @self: a tab overview
  * @secondary_menu: (nullable): a menu model
  *
@@ -2377,7 +2374,7 @@ adw_tab_overview_set_secondary_menu (AdwTabOverview *self,
 }
 
 /**
- * adw_tab_overview_get_show_start_title_buttons: (attributes org.gtk.Method.get_property=show-start-title-buttons)
+ * adw_tab_overview_get_show_start_title_buttons:
  * @self: a tab overview
  *
  * Gets whether start title buttons are shown in @self's header bar.
@@ -2395,7 +2392,7 @@ adw_tab_overview_get_show_start_title_buttons (AdwTabOverview *self)
 }
 
 /**
- * adw_tab_overview_set_show_start_title_buttons: (attributes org.gtk.Method.set_property=show-start-title-buttons)
+ * adw_tab_overview_set_show_start_title_buttons:
  * @self: a tab overview
  * @show_start_title_buttons: whether to show start title buttons
  *
@@ -2425,7 +2422,7 @@ adw_tab_overview_set_show_start_title_buttons (AdwTabOverview *self,
 }
 
 /**
- * adw_tab_overview_get_show_end_title_buttons: (attributes org.gtk.Method.get_property=show-end-title-buttons)
+ * adw_tab_overview_get_show_end_title_buttons:
  * @self: a tab overview
  *
  * Gets whether end title buttons are shown in @self's header bar.
@@ -2443,7 +2440,7 @@ adw_tab_overview_get_show_end_title_buttons (AdwTabOverview *self)
 }
 
 /**
- * adw_tab_overview_set_show_end_title_buttons: (attributes org.gtk.Method.set_property=show-end-title-buttons)
+ * adw_tab_overview_set_show_end_title_buttons:
  * @self: a tab overview
  * @show_end_title_buttons: whether to show end title buttons
  *
@@ -2543,7 +2540,7 @@ adw_tab_overview_get_extra_drag_preferred_action (AdwTabOverview *self)
 }
 
 /**
- * adw_tab_overview_get_extra_drag_preload: (attributes org.gtk.Method.get_property=extra-drag-preload)
+ * adw_tab_overview_get_extra_drag_preload:
  * @self: a tab overview
  *
  * Gets whether drop data should be preloaded on hover.
@@ -2561,7 +2558,7 @@ adw_tab_overview_get_extra_drag_preload (AdwTabOverview *self)
 }
 
 /**
- * adw_tab_overview_set_extra_drag_preload: (attributes org.gtk.Method.set_property=extra-drag-preload)
+ * adw_tab_overview_set_extra_drag_preload:
  * @self: a tab overview
  * @preload: whether to preload drop data
  *
