@@ -27,7 +27,6 @@ struct _AdwTabThumbnail
   GtkPicture *picture;
   GtkWidget *icon_stack;
   GtkImage *icon;
-  GtkSpinner *spinner;
   GtkImage *indicator_icon;
   GtkWidget *indicator_btn;
   GtkWidget *close_btn;
@@ -91,16 +90,6 @@ update_tooltip (AdwTabThumbnail *self)
                                  adw_tab_page_get_title (page));
 }
 
-static void
-update_spinner (AdwTabThumbnail *self)
-{
-  gboolean loading = self->page && adw_tab_page_get_loading (self->page);
-  gboolean mapped = gtk_widget_get_mapped (GTK_WIDGET (self));
-
-  /* Don't use CPU when not needed */
-  gtk_spinner_set_spinning (self->spinner, loading && mapped);
-}
-
 static void
 update_icon (AdwTabThumbnail *self)
 {
@@ -118,7 +107,6 @@ static void
 update_loading (AdwTabThumbnail *self)
 {
   update_icon (self);
-  update_spinner (self);
   set_style_class (GTK_WIDGET (self), "loading",
                    adw_tab_page_get_loading (self->page));
 }
@@ -144,12 +132,10 @@ update_indicator (AdwTabThumbnail *self)
   set_style_class (GTK_WIDGET (self), "indicator", indicator != NULL);
 }
 
-static gboolean
+static void
 close_idle_cb (AdwTabThumbnail *self)
 {
   adw_tab_view_close_page (self->view, self->page);
-
-  return G_SOURCE_REMOVE;
 }
 
 static void
@@ -161,15 +147,13 @@ close_clicked_cb (AdwTabThumbnail *self)
   /* When animations are disabled, we don't want to immediately remove the
    * whole tab mid-click. Instead, defer it until the click has happened.
    */
-  g_idle_add ((GSourceFunc) close_idle_cb, self);
+  g_idle_add_once ((GSourceOnceFunc) close_idle_cb, self);
 }
 
-static gboolean
+static void
 unpin_idle_cb (AdwTabThumbnail *self)
 {
   adw_tab_view_set_page_pinned (self->view, self->page, FALSE);
-
-  return G_SOURCE_REMOVE;
 }
 
 static void
@@ -181,7 +165,7 @@ unpin_clicked_cb (AdwTabThumbnail *self)
   /* When animations are disabled, we don't want to immediately unpin the
    * whole tab mid-click. Instead, defer it until the click has happened.
    */
-  g_idle_add ((GSourceFunc) unpin_idle_cb, self);
+  g_idle_add_once ((GSourceOnceFunc) unpin_idle_cb, self);
 }
 
 static void
@@ -262,26 +246,6 @@ fade_animation_value_cb (double           value,
   gtk_widget_set_opacity (self->needs_attention_revealer, value);
 }
 
-static void
-adw_tab_thumbnail_map (GtkWidget *widget)
-{
-  AdwTabThumbnail *self = ADW_TAB_THUMBNAIL (widget);
-
-  GTK_WIDGET_CLASS (adw_tab_thumbnail_parent_class)->map (widget);
-
-  update_spinner (self);
-}
-
-static void
-adw_tab_thumbnail_unmap (GtkWidget *widget)
-{
-  AdwTabThumbnail *self = ADW_TAB_THUMBNAIL (widget);
-
-  GTK_WIDGET_CLASS (adw_tab_thumbnail_parent_class)->unmap (widget);
-
-  update_spinner (self);
-}
-
 static void
 measure_pinned_tab (AdwGizmo       *gizmo,
                     GtkOrientation  orientation,
@@ -492,9 +456,6 @@ adw_tab_thumbnail_class_init (AdwTabThumbnailClass *klass)
   object_class->get_property = adw_tab_thumbnail_get_property;
   object_class->set_property = adw_tab_thumbnail_set_property;
 
-  widget_class->map = adw_tab_thumbnail_map;
-  widget_class->unmap = adw_tab_thumbnail_unmap;
-
   props[PROP_VIEW] =
     g_param_spec_object ("view", NULL, NULL,
                          ADW_TYPE_TAB_VIEW,
@@ -548,7 +509,6 @@ adw_tab_thumbnail_class_init (AdwTabThumbnailClass *klass)
   gtk_widget_class_bind_template_child (widget_class, AdwTabThumbnail, picture);
   gtk_widget_class_bind_template_child (widget_class, AdwTabThumbnail, icon_stack);
   gtk_widget_class_bind_template_child (widget_class, AdwTabThumbnail, icon);
-  gtk_widget_class_bind_template_child (widget_class, AdwTabThumbnail, spinner);
   gtk_widget_class_bind_template_child (widget_class, AdwTabThumbnail, indicator_icon);
   gtk_widget_class_bind_template_child (widget_class, AdwTabThumbnail, indicator_btn);
   gtk_widget_class_bind_template_child (widget_class, AdwTabThumbnail, close_btn);
@@ -630,7 +590,6 @@ adw_tab_thumbnail_set_page (AdwTabThumbnail *self,
     gtk_picture_set_paintable (GTK_PICTURE (self->picture), paintable);
 
     update_tooltip (self);
-    update_spinner (self);
     update_icon (self);
     update_indicator (self);
     update_loading (self);
