@@ -74,6 +74,7 @@ typedef struct {
   AdwAnimation *show_animation;
   AdwAnimation *hide_animation;
 
+  gulong shown_id;
   gulong dismissed_id;
   gboolean postponing;
 } ToastInfo;
@@ -107,6 +108,8 @@ static GtkBuildableIface *parent_buildable_iface;
 static void
 free_toast_info (ToastInfo *info)
 {
+  if (info->shown_id && info->show_animation)
+    g_signal_handler_disconnect (info->show_animation, info->shown_id);
   if (info->dismissed_id && info->toast)
     g_signal_handler_disconnect (info->toast, info->dismissed_id);
 
@@ -253,8 +256,8 @@ show_toast (AdwToastOverlay *self,
                              self->hiding_toasts ? REPLACE_DURATION : SHOW_DURATION,
                              target);
 
-  g_signal_connect_swapped (info->show_animation, "done",
-                            G_CALLBACK (show_done_cb), info);
+  info->shown_id = g_signal_connect_swapped (info->show_animation, "done",
+                                             G_CALLBACK (show_done_cb), info);
 
   adw_animation_play (info->show_animation);
 }
@@ -490,11 +493,12 @@ adw_toast_overlay_class_init (AdwToastOverlayClass *klass)
   object_class->set_property = adw_toast_overlay_set_property;
 
   widget_class->compute_expand = adw_widget_compute_expand;
+  widget_class->get_request_mode = adw_widget_get_request_mode;
   widget_class->measure = adw_toast_overlay_measure;
   widget_class->size_allocate = adw_toast_overlay_size_allocate;
 
   /**
-   * AdwToastOverlay:child: (attributes org.gtk.Property.get=adw_toast_overlay_get_child org.gtk.Property.set=adw_toast_overlay_set_child)
+   * AdwToastOverlay:child:
    *
    * The child widget.
    */
@@ -557,7 +561,7 @@ adw_toast_overlay_new (void)
 }
 
 /**
- * adw_toast_overlay_get_child: (attributes org.gtk.Method.get_property=child)
+ * adw_toast_overlay_get_child:
  * @self: a toast overlay
  *
  * Gets the child widget of @self.
@@ -573,7 +577,7 @@ adw_toast_overlay_get_child (AdwToastOverlay *self)
 }
 
 /**
- * adw_toast_overlay_set_child: (attributes org.gtk.Method.set_property=child)
+ * adw_toast_overlay_set_child:
  * @self: a toast overlay
  * @child: (nullable): the child widget
  *
@@ -586,6 +590,9 @@ adw_toast_overlay_set_child (AdwToastOverlay *self,
   g_return_if_fail (ADW_IS_TOAST_OVERLAY (self));
   g_return_if_fail (child == NULL || GTK_IS_WIDGET (child));
 
+  if (child)
+    g_return_if_fail (gtk_widget_get_parent (child) == NULL);
+
   if (self->child == child)
     return;
 
